"""汎用的なモデルを定義するモジュールです。"""

from mipac.types.common import IID


class ID:
    def __init__(self, *, raw_id: IID) -> None:
        self.__raw_id: IID = raw_id

    @property
    def id(self) -> str:
        """The id of the object"""
        return self.__raw_id["id"]



from __future__ import annotations
from datetime import datetime

from typing import TYPE_CHECKING, Any

from mipac.models.lite.user import PartialUser
from mipac.types.drive import IDriveStatus
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.drive.files import ClientFileManager
    from mipac.manager.drive.folders import ClientFolderManager
    from mipac.types import IFile, IFileProperties, IFolder


__all__ = ["FileProperties", "File", "Folder"]


class DriveStatus:
    def __init__(self, raw_drive_status: IDriveStatus, *, client: ClientManager) -> None:
        self.__raw_drive_status: IDriveStatus = raw_drive_status
        self.__client: ClientManager = client

    @property
    def capacity(self) -> int:
        """Total capacity of the drive in bytes

        Returns
        -------
        int
            Total capacity of the drive in bytes
        """
        return self.__raw_drive_status["capacity"]

    @property
    def usage(self) -> int:
        """Total usage of the drive in bytes

        Returns
        -------
        int
            Total usage of the drive in bytes
        """
        return self.__raw_drive_status["usage"]

    def _get(self, key: str) -> Any | None:
        """You can access the raw response data directly by specifying the key

        Returns
        -------
        Any | None
            raw response data
        """
        return self.__raw_drive_status.get(key)


class FileProperties:
    def __init__(self, raw_properties: IFileProperties) -> None:
        self.__raw_properties: IFileProperties = raw_properties

    @property
    def width(self) -> int | None:
        """ファイルの幅

        Returns
        -------
        int | None
            ファイルの幅
        """
        return self.__raw_properties.get("width")

    @property
    def height(self) -> int | None:
        """ファイルの高さ

        Returns
        -------
        int | None
            ファイルの高さ
        """
        return self.__raw_properties.get("height")

    @property
    def orientation(self) -> int | None:
        return self.__raw_properties.get("orientation")

    @property
    def avg_color(self) -> str | None:
        return self.__raw_properties.get("avg_color")

    def _get(self, key: str) -> Any | None:
        """You can access the raw response data directly by specifying the key

        Returns
        -------
        Any | None
            raw response data
        """
        return self.__raw_properties.get(key)


class Folder:
    def __init__(self, raw_folder: IFolder, client: ClientManager):
        self.__raw_folder: IFolder = raw_folder
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """フォルダーのID

        Returns
        -------
        str
            フォルダーのID
        """
        return self.__raw_folder["id"]

    @property
    def created_at(self) -> datetime:
        """フォルダーが作成された日時

        Returns
        -------
        datetime
            フォルダーが作成された日時
        """
        return str_to_datetime(self.__raw_folder["created_at"])

    @property
    def name(self) -> str:
        """フォルダー名

        Returns
        -------
        str
            フォルダー名
        """
        return self.__raw_folder["name"]

    @property
    def parent_id(self) -> str | None:
        """親フォルダーのID

        Returns
        -------
        str | None
            親フォルダーのID
        """
        return self.__raw_folder["parent_id"]

    @property
    def folders_count(self) -> int | None:
        """フォルダー内のフォルダーの数

        Returns
        -------
        int | None
            フォルダー内のフォルダーの数
        """
        return self.__raw_folder.get("folders_count")

    @property
    def files_count(self) -> int | None:
        """フォルダー内のファイルの数

        Returns
        -------
        int | None
            フォルダー内のファイルの数
        """
        return self.__raw_folder.get("files_count")

    @property
    def parent(self) -> Folder | None:
        """親フォルダー

        Returns
        -------
        Folder | None
            親フォルダー
        """
        return (
            Folder(self.__raw_folder["parent"], client=self.__client)
            if "parent" in self.__raw_folder and self.__raw_folder["parent"]
            else None
        )

    def _get(self, key: str) -> Any | None:
        """You can access the raw response data directly by specifying the key

        Returns
        -------
        Any | None
            raw response data
        """
        return self.__raw_folder.get(key)

    @property
    def api(self) -> ClientFolderManager:
        return self.__client.drive._create_client_folder_manager(folder_id=self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Folder) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class File:
    def __init__(self, raw_file: IFile, *, client: ClientManager):
        self.__raw_file: IFile = raw_file
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """ファイルのID

        Returns
        -------
        str
            ファイルのID
        """
        return self.__raw_file["id"]

    @property
    def created_at(self) -> datetime:
        """ファイルの作成日時

        Returns
        -------
        datetime
            ファイルの作成日時
        """
        return str_to_datetime(self.__raw_file["created_at"])

    @property
    def name(self) -> str:
        """拡張子付きのファイル名

        Returns
        -------
        str
            拡張子付きのファイル名
        """
        return self.__raw_file["name"]

    @property
    def type(self) -> str:
        """ファイルの種類 (MIME タイプ)

        Returns
        -------
        str
            ファイルの種類 (MIME タイプ)
        """
        return self.__raw_file["type"]

    @property
    def md5(self) -> str:
        """ファイルのMD5ハッシュ

        Returns
        -------
        str
            ファイルのMD5ハッシュ
        """
        return self.__raw_file["md5"]

    @property
    def size(self) -> int:
        """ファイルのサイズ

        Returns
        -------
        int
            ファイルのサイズ
        """
        return self.__raw_file["size"]

    @property
    def is_sensitive(self) -> bool:
        """ファイルが閲覧注意（NSFW）であるか

        Returns
        -------
        bool
            ファイルが閲覧注意であるか
        """
        return self.__raw_file["is_sensitive"]

    @property
    def blurhash(self) -> str | None:
        """ファイルのBlurHash

        Returns
        -------
        str | None
            ファイルのBlurHash
        """
        return self.__raw_file["blurhash"]

    @property
    def properties(self) -> FileProperties:
        """ファイルのプロパティ

        Returns
        -------
        FileProperties
            ファイルのプロパティ
        """
        return FileProperties(self.__raw_file["properties"])

    @property
    def url(self) -> str:
        """ファイルのURL

        Returns
        -------
        str
            ファイルのURL
        """
        return self.__raw_file["url"]

    @property
    def thumbnail_url(self) -> str | None:
        """ファイルのサムネイルURL

        Returns
        -------
        str | None
            ファイルのサムネイルURL
        """
        return self.__raw_file["thumbnail_url"]

    @property
    def comment(self) -> str | None:
        """ファイルのコメント

        Returns
        -------
        str | None
            ファイルのコメント
        """
        return self.__raw_file["comment"]

    @property
    def folder_id(self) -> str | None:
        """ファイルが存在するフォルダのID

        Returns
        -------
        str | None
            ファイルが存在するフォルダのID
        """
        return self.__raw_file["folder_id"]

    @property
    def folder(self) -> Folder | None:
        """ファイルが存在するフォルダ

        Returns
        -------
        Folder | None
            ファイルが存在するフォルダ
        """
        return (
            Folder(self.__raw_file["folder"], client=self.__client)
            if "folder" in self.__raw_file and self.__raw_file["folder"]
            else None
        )

    @property
    def user_id(self) -> str | None:
        """ファイルをアップロードしたユーザーのID

        Returns
        -------
        str | None
            ファイルをアップロードしたユーザーのID
        """
        return self.__raw_file["user_id"]

    @property
    def user(self) -> PartialUser | None:
        """ファイルをアップロードしたユーザー

        Returns
        -------
        PartialUser | None
            ファイルをアップロードしたユーザー
        """
        return (
            PartialUser(self.__raw_file["user"], client=self.__client)
            if "user" in self.__raw_file and self.__raw_file["user"]
            else None
        )

    def _get(self, key: str) -> Any | None:
        """生のレスポンスデータに直接アクセスすることができます

        Returns
        -------
        Any | None
            生のレスポンスデータ
        """
        return self.__raw_file.get(key)

    @property
    def api(self) -> ClientFileManager:
        """このファイルに対する操作を行うためのAPIクライアント

        Returns
        -------
        ClientFileManager
            このファイルに対する操作を行うためのAPIクライアント
        """
        return self.__client.drive._create_client_file_manager(file_id=self.id)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, File) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.username import UsernameAvailable
from mipac.types.username import IUsernameAvailable

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class UsernameActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def available(self, username: str) -> UsernameAvailable:
        data = {"username": username}
        res: IUsernameAvailable = await self.__session.request(
            Route("POST", "/api/username/available"), json=data, auth=True
        )
        return UsernameAvailable(res)



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, override

from mipac.abstract.action import AbstractAction
from mipac.errors.base import APIError
from mipac.file import MiFile
from mipac.http import HTTPClient, Route
from mipac.models.clip import Clip
from mipac.models.drive import File
from mipac.models.note import Note, NoteReaction, NoteState, NoteTranslateResult
from mipac.models.poll import MiPoll, Poll
from mipac.types.clip import IClip
from mipac.types.note import ICreatedNote, INote, INoteState, INoteTranslateResult, INoteVisibility
from mipac.types.reaction import IReactionAcceptance
from mipac.utils.cache import cache
from mipac.utils.format import remove_dict_empty
from mipac.utils.pagination import Pagination
from mipac.utils.util import check_multi_arg, deprecated

if TYPE_CHECKING:
    from mipac.client import ClientManager

__all__ = ["NoteActions"]


def create_note_body(
    text: str | None = None,
    visibility: INoteVisibility = "public",
    visible_user_ids: list[str] | None = None,
    cw: str | None = None,
    local_only: bool = False,
    reaction_acceptance: IReactionAcceptance = None,
    extract_mentions: bool = True,
    extract_hashtags: bool = True,
    extract_emojis: bool = True,
    reply_id: str | None = None,
    renote_id: str | None = None,
    channel_id: str | None = None,
    files: list[MiFile | File | str] | None = None,
    media_ids: list[str] | None = None,
    poll: MiPoll | None = None,
):
    text = text or None
    body = {
        "visibility": visibility,
        "visibleUserIds": visible_user_ids,
        "text": text,
        "mediaIds": media_ids,
        "cw": cw,
        "localOnly": local_only,
        "reactionAcceptance": reaction_acceptance,
        "noExtractMentions": not extract_mentions,
        "noExtractHashtags": not extract_hashtags,
        "noExtractEmojis": not extract_emojis,
        "replyId": reply_id,
        "renoteId": renote_id,
        "channelId": channel_id,
    }
    if not check_multi_arg(text, files, renote_id, poll):
        raise ValueError("To send a note, one of content, file_ids, renote_id or poll is required")

    if poll and type(Poll):
        poll_data = remove_dict_empty(
            {
                "choices": poll.choices,
                "multiple": poll.multiple,
                "expiresAt": poll.expires_at,
                "expiredAfter": poll.expired_after,
            }
        )
        body["poll"] = poll_data
    if files:
        file_ids = []
        for file in files:
            if isinstance(file, MiFile):
                file_ids.append(file.file_id)
            elif isinstance(file, File):
                file_ids.append(file.id)
            elif isinstance(file, str):
                file_ids.append(file)
            else:
                raise ValueError("files must be MiFile or str or File")
        body["fileIds"] = file_ids

    return remove_dict_empty(body)


class SharedNoteActions(AbstractAction):
    def __init__(
        self,
        session: HTTPClient,
        client: ClientManager,
    ):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    @cache(group="get_note_children")
    async def get_children(
        self,
        limit: int = 100,
        since_id: str | None = None,
        untilId: str | None = None,
        *,
        note_id: str,
    ) -> list[Note]:
        data = {
            "noteId": note_id,
            "limit": limit,
            "sinceId": since_id,
            "untilId": untilId,
        }

        notes: list[INote] = await self._session.request(
            Route("POST", "/api/notes/children"), json=data
        )
        return [Note(note, self._client) for note in notes]

    @cache(group="get_note_children", override=True)
    async def fetch_children(
        self,
        limit: int = 100,
        since_id: str | None = None,
        untilId: str | None = None,
        *,
        note_id: str,
    ) -> list[Note]:
        """Get children of the note.
        update the cache of the :py:meth:`mipac.actions.note.ClientNoteActions.get_children` method

        Endpoint: `/api/notes/children`

        Parameters
        ----------
        limit : int, default=100
            limit
        since_id : str | None, default=None
            Since ID
        untilId : str | None, default=None
            Until ID
        note_id : str | None, default=None
            note id

        Returns
        -------
        list[Note]
            Children of the note
        """
        return await self.get_children(
            limit=limit, since_id=since_id, untilId=untilId, note_id=note_id
        )

    async def get_all_children(
        self,
        limit: int = 10,
        since_id: str | None = None,
        untilId: str | None = None,
        *,
        note_id: str,
    ) -> AsyncGenerator[Note, None]:
        """Get all children of the note

        Endpoint: `/api/notes/children`

        Parameters
        ----------
        since_id : str | None, default=None
            Since ID
        untilId : str | None, default=None
            Until ID
        note_id : str | None, default=None
            note id

        Returns
        -------
        AsyncGenerator[Note, None]
            Children of the note
        """
        data = {
            "noteId": note_id,
            "limit": limit,
            "sinceId": since_id,
            "untilId": untilId,
        }

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/notes/children"), json=data
        )

        while pagination.is_final is False:
            res_notes = await pagination.next()
            for note in res_notes:
                yield Note(note, self._client)

    async def get_clips(self, *, note_id: str) -> list[Clip]:
        """Get the clips of the note

        Endpoint: `/api/notes/clips`

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        list[Clip]
            Clips of the note
        """
        data = {"noteId": note_id}
        res: list[IClip] = await self._session.request(
            Route("POST", "/api/notes/clips"), json=data, auth=True
        )
        return [Clip(clip, client=self._client) for clip in res]

    async def get_conversation(
        self, limit: int = 10, offset: int = 0, *, note_id: str
    ) -> list[Note]:
        """Get the conversation of the note

        Endpoint: `/api/notes/conversation`

        Parameters
        ----------
        limit : int, default=10
            limit
        offset : int, default=0
            offset
        note_id : str | None, default=None
            note id

        Returns
        -------
        list[Note]
            Notes of the conversation
        """
        data = {"noteId": note_id, "limit": limit, "offset": offset}
        res: list[INote] = await self._session.request(
            Route("POST", "/api/notes/conversation"), json=data, auth=True
        )
        return [Note(note, client=self._client) for note in res]

    async def delete(self, *, note_id: str) -> bool:
        """Delete a note

        Endpoint: `/api/notes/delete`

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        bool
            success or not
        """
        data = {"noteId": note_id}
        res = await self._session.request(Route("POST", "/api/notes/delete"), json=data, auth=True)
        return bool(res)

    async def get_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        note_id: str,
    ) -> list[NoteReaction]:
        return await self._client.note.reaction.action.get_reactions(
            type=type, note_id=note_id, limit=limit, since_id=since_id, until_id=until_id
        )

    async def fetch_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        note_id: str,
    ) -> list[NoteReaction]:
        return await self._client.note.reaction.action.fetch_reactions(
            note_id=note_id, type=type, limit=limit, since_id=since_id, until_id=until_id
        )

    async def get_renotes(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        note_id: str,
    ) -> list[Note]:
        """Get renote of the note

        Endpoint: `/api/notes/renotes`

        Parameters
        ----------
        limit : int, default=10
            limit
        since_id : str | None, default=None
            Since ID
        until_id : str | None, default=None
            Until ID
        note_id : str | None, default=None
            note id

        Returns
        -------
        list[Note]
            Renotes of the note
        """
        data = {
            "noteId": note_id,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
        }

        res: list[INote] = await self._session.request(
            Route("POST", "/api/notes/renotes"), json=data, auth=True
        )
        return [Note(note, client=self._client) for note in res]

    async def get_replies(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        *,
        note_id: str,
    ) -> list[Note]:
        """Get replies to the note

        Endpoint: `/api/notes/replies`

        Parameters
        ----------
        since_id : str | None, default=None
            since id
        until_id : str | None, default=None
            until id
        limit : int, default=10
            limit
        note_id: str | None, default=None
            note id

        Returns
        -------
        list[Note]
            replies
        """
        data = {
            "noteId": note_id,
            "sinceId": since_id,
            "untilId": until_id,
            "limit": limit,
        }
        res: list[INote] = await self._session.request(
            Route("POST", "/api/notes/replies"), json=data, auth=True
        )
        return [Note(note, client=self._client) for note in res]

    async def get_all_replies(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        *,
        note_id: str,
    ) -> AsyncGenerator[Note, None]:
        """Get replies to the note

        Endpoint: `/api/notes/replies`

        Parameters
        ----------
        since_id : str | None, default=None
            since id
        until_id : str | None, default=None
            until id
        note_id: str | None, default=None
            note id

        Returns
        -------
        AsyncGenerator[Note, None]
            replies
        """
        body = {"noteId": note_id, "sinceId": since_id, "untilId": until_id, "limit": limit}

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/notes/replies"), json=body
        )

        while pagination.is_final is False:
            res_notes = await pagination.next()
            for res_note in res_notes:
                yield Note(res_note, client=self._client)

    @cache(group="get_note_state")
    async def get_state(self, *, note_id: str) -> NoteState:
        """Get the state of the note

        Endpoint: `/api/notes/state`

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        NoteState
            Note state
        """
        data = {"noteId": note_id}
        res: INoteState = await self._session.request(
            Route("POST", "/api/notes/state"), auth=True, json=data
        )
        return NoteState(res)

    @cache(group="get_note_state", override=True)
    async def fetch_state(self, *, note_id: str) -> NoteState:
        """Get the state of the note.

        update the cache of the :py:meth:`mipac.actions.note.ClientNoteActions.get_state` method

        Endpoint: `/api/notes/state`

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        NoteState
            Note state
        """
        return await self.get_state(note_id=note_id)

    async def add_clips(self, clip_id: str, *, note_id: str) -> bool:
        """Add a note to the clip

        Endpoint: `/api/clips/add-note`

        Parameters
        ----------
        note_id : str | None, default=None
            note id
        clip_id : str
            clip id

        Returns
        -------
        bool
            success or not
        """
        data = {"noteId": note_id, "clipId": clip_id}
        return bool(
            await self._session.request(Route("POST", "/api/clips/add-note"), json=data, auth=True)
        )

    async def create_renote(self, *, note_id: str) -> Note:
        """Renote a note

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        Note
            Renoted note
        """
        body = create_note_body(renote_id=note_id)
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            auth=True,
            lower=True,
        )
        return Note(res["created_note"], client=self._client)

    async def renote(
        self,
        text: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        channel_id: str | None = None,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
        *,
        renote_id: str,
    ):
        """Renote a note

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        text : str | None, default=None
            text
        visibility : INoteVisibility, default='public'
            Disclosure range
        visible_user_ids : list[str] | None, default=None
            List of users to be published
        cw : str | None, default=None
            Text to be displayed when warning is given
        local_only : bool, default=False
            Whether to show only locally or not
        reaction_acceptance : IReactionAcceptance, default=None
            Reaction acceptance setting
        extract_mentions : bool, default=True
            Whether to expand the mention
        extract_hashtags : bool, default=True
            Whether to expand the hashtag
        extract_emojis : bool, default=True
            Whether to expand the emojis
        channel_id : str | None, default=None
            Channel ID
        files : list[MiFile | File | str] | None, default=None
            The ID list of files to be attached
        poll : MiPoll | None, default=None
        """
        body = create_note_body(
            text=text,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            renote_id=renote_id,
            channel_id=channel_id,
            files=files,
            poll=poll,
        )

        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            auth=True,
            lower=True,
        )

        return Note(res["created_note"], client=self._client)

    async def reply(
        self,
        text: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
        *,
        reply_id: str,
    ) -> Note:
        body = create_note_body(
            text=text,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            reply_id=reply_id,
            files=files,
            poll=poll,
        )
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            lower=True,
            auth=True,
        )
        return Note(res["created_note"], client=self._client)

    async def create_quote(
        self,
        content: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
        *,
        note_id: str,
    ) -> Note:
        """Create a note quote.

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        content: str | None, default=None
            text
        visibility: INoteVisibility, default='public'
            Disclosure range
        visible_user_ids: list[str] | None, default=None
            List of users to be published
        cw: str | None, default=None
            Text to be displayed when warning is given
        local_only: bool, default=False
            Whether to show only locally or not
        extract_mentions: bool, default=True
            Whether to expand the mention
        extract_hashtags: bool, default=True
            Whether to expand the hashtag
        extract_emojis: bool, default=True
            Whether to expand the emojis
        files: list[MiFile | File | str] | None, default=None
            The ID list of files to be attached
        poll: MiPoll | None, default=None
            Questionnaire to be created
        note_id: str | None, default=None
            Note IDs to target for renote and citations
        """
        body = create_note_body(
            text=content,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            renote_id=note_id,
            files=files,
            poll=poll,
        )
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            auth=True,
            lower=True,
        )

        return Note(res["created_note"], client=self._client)

    @cache(group="translate_note")
    async def translate(
        self,
        target_lang: str = "en-US",
        *,
        note_id: str,
    ) -> NoteTranslateResult:
        """Translate a note

        Endpoint: `/api/notes/translate`

        Parameters
        ----------
        note_id : str | None, default=None
            Note ID to target for translation
        target_lang : str, default='en'
            Target language

        Returns
        -------
        NoteTranslateResult
            Translated result
        """
        data = {"noteId": note_id, "targetLang": target_lang}
        res: INoteTranslateResult = await self._session.request(
            Route("POST", "/api/notes/translate"), json=data, auth=True
        )
        if isinstance(res, dict):
            return NoteTranslateResult(res)
        APIError(f"Translate Error: {res}", res if isinstance(res, int) else 204).raise_error()

    async def un_renote(self, *, note_id: str) -> bool:
        """Releases the note renote for the specified Id

        Parameters
        ----------
        note_id : str | None, optional
            Target note Id., by default None

        Returns
        -------
        bool
            Whether the release was successful
        """
        body = {"noteId": note_id}
        res: bool = await self._session.request(
            Route("POST", "/api/notes/unrenote"), auth=True, json=body
        )
        return res


class ClientNoteActions(SharedNoteActions):
    def __init__(
        self,
        note_id: str,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        super().__init__(session, client)
        self._note_id: str = note_id

    @override
    async def get_children(
        self,
        limit: int = 100,
        since_id: str | None = None,
        untilId: str | None = None,
    ) -> list[Note]:
        return await super().get_children(
            limit=limit, since_id=since_id, untilId=untilId, note_id=self._note_id
        )

    @override
    async def fetch_children(
        self,
        limit: int = 100,
        since_id: str | None = None,
        untilId: str | None = None,
    ) -> list[Note]:
        """Get children of the note.
        update the cache of the :py:meth:`mipac.actions.note.ClientNoteActions.get_children` method

        Endpoint: `/api/notes/children`

        Parameters
        ----------
        limit : int, default=100
            limit
        since_id : str | None, default=None
            Since ID
        untilId : str | None, default=None
            Until ID

        Returns
        -------
        list[Note]
            Children of the note
        """
        return await super().fetch_children(
            limit=limit, since_id=since_id, untilId=untilId, note_id=self._note_id
        )

    @override
    async def get_all_children(
        self,
        limit: int = 10,
        since_id: str | None = None,
        untilId: str | None = None,
    ) -> AsyncGenerator[Note, None]:
        """Get all children of the note

        Endpoint: `/api/notes/children`

        Parameters
        ----------
        since_id : str | None, default=None
            Since ID
        untilId : str | None, default=None
            Until ID

        Returns
        -------
        AsyncGenerator[Note, None]
            Children of the note
        """
        async for i in super().get_all_children(
            limit=limit, since_id=since_id, untilId=untilId, note_id=self._note_id
        ):
            yield i

    @override
    async def get_clips(self) -> list[Clip]:
        """Get the clips of the note

        Endpoint: `/api/notes/clips`

        Returns
        -------
        list[Clip]
            Clips of the note
        """
        return await super().get_clips(note_id=self._note_id)

    @override
    async def get_conversation(self, limit: int = 10, offset: int = 0) -> list[Note]:
        """Get the conversation of the note

        Endpoint: `/api/notes/conversation`

        Parameters
        ----------
        limit : int, default=10
            limit
        offset : int, default=0
            offset

        Returns
        -------
        list[Note]
            Notes of the conversation
        """
        return await super().get_conversation(limit=limit, offset=offset, note_id=self._note_id)

    @override
    async def delete(self) -> bool:
        """Delete a note

        Endpoint: `/api/notes/delete`

        Parameters
        ----------
        note_id : str | None, default=None
            note id

        Returns
        -------
        bool
            success or not
        """
        return await super().delete(note_id=self._note_id)

    @override
    async def get_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> list[NoteReaction]:
        return await super().get_reactions(
            type=type,
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            note_id=self._note_id,
        )

    @override
    async def fetch_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> list[NoteReaction]:
        return await super().fetch_reactions(
            type=type,
            limit=limit,
            since_id=since_id,
            until_id=until_id,
            note_id=self._note_id,
        )

    @override
    async def get_renotes(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> list[Note]:
        """Get renote of the note

        Endpoint: `/api/notes/renotes`

        Parameters
        ----------
        limit : int, default=10
            limit
        since_id : str | None, default=None
            Since ID
        until_id : str | None, default=None
            Until ID

        Returns
        -------
        list[Note]
            Renotes of the note
        """

        return await super().get_renotes(
            limit=limit, since_id=since_id, until_id=until_id, note_id=self._note_id
        )

    @override
    async def get_replies(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
    ) -> list[Note]:
        """Get replies to the note

        Endpoint: `/api/notes/replies`

        Parameters
        ----------
        since_id : str | None, default=None
            since id
        until_id : str | None, default=None
            until id
        limit : int, default=10
            limit

        Returns
        -------
        list[Note]
            replies
        """
        return await super().get_replies(
            since_id=since_id, until_id=until_id, limit=limit, note_id=self._note_id
        )

    @override
    async def get_all_replies(
        self,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
    ) -> AsyncGenerator[Note, None]:
        """Get replies to the note

        Endpoint: `/api/notes/replies`

        Parameters
        ----------
        since_id : str | None, default=None
            since id
        until_id : str | None, default=None
            until id

        Returns
        -------
        AsyncGenerator[Note, None]
            replies
        """

        async for i in super().get_all_replies(
            since_id=since_id, until_id=until_id, limit=limit, note_id=self._note_id
        ):
            yield i

    @override
    async def get_state(self) -> NoteState:
        """Get the state of the note

        Endpoint: `/api/notes/state`

        Returns
        -------
        NoteState
            Note state
        """

        return await super().get_state(note_id=self._note_id)

    @override
    async def fetch_state(self) -> NoteState:
        """Get the state of the note.

        update the cache of the :py:meth:`mipac.actions.note.ClientNoteActions.get_state` method

        Endpoint: `/api/notes/state`

        Returns
        -------
        NoteState
            Note state
        """
        return await super().fetch_state(note_id=self._note_id)

    async def add_clips(self, clip_id: str) -> bool:
        """Add a note to the clip

        Endpoint: `/api/clips/add-note`

        Parameters
        ----------
        clip_id : str
            clip id

        Returns
        -------
        bool
            success or not
        """
        return await super().add_clips(clip_id=clip_id, note_id=self._note_id)

    @override
    async def create_renote(self) -> Note:
        """Renote a note

        Endpoint: `/api/notes/create`

        Returns
        -------
        Note
            Renoted note
        """

        return await super().create_renote(note_id=self._note_id)

    @override
    async def renote(
        self,
        text: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        channel_id: str | None = None,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
    ):
        """Renote a note

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        text : str | None, default=None
            text
        visibility : INoteVisibility, default='public'
            Disclosure range
        visible_user_ids : list[str] | None, default=None
            List of users to be published
        cw : str | None, default=None
            Text to be displayed when warning is given
        local_only : bool, default=False
            Whether to show only locally or not
        reaction_acceptance : IReactionAcceptance, default=None
            Reaction acceptance setting
        extract_mentions : bool, default=True
            Whether to expand the mention
        extract_hashtags : bool, default=True
            Whether to expand the hashtag
        extract_emojis : bool, default=True
            Whether to expand the emojis
        channel_id : str | None, default=None
            Channel ID
        files : list[MiFile | File | str] | None, default=None
            The ID list of files to be attached
        poll : MiPoll | None, default=None
        """

        return await super().renote(
            text=text,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            channel_id=channel_id,
            files=files,
            poll=poll,
            renote_id=self._note_id,
        )

    @override
    async def reply(
        self,
        text: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
    ) -> Note:
        return await super().reply(
            text=text,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            files=files,
            poll=poll,
            reply_id=self._note_id,
        )

    @override
    async def create_quote(
        self,
        content: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,
        extract_hashtags: bool = True,
        extract_emojis: bool = True,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
    ) -> Note:
        """Create a note quote.

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        content: str | None, default=None
            text
        visibility: INoteVisibility, default='public'
            Disclosure range
        visible_user_ids: list[str] | None, default=None
            List of users to be published
        cw: str | None, default=None
            Text to be displayed when warning is given
        local_only: bool, default=False
            Whether to show only locally or not
        extract_mentions: bool, default=True
            Whether to expand the mention
        extract_hashtags: bool, default=True
            Whether to expand the hashtag
        extract_emojis: bool, default=True
            Whether to expand the emojis
        files: list[MiFile | File | str] | None, default=None
            The ID list of files to be attached
        poll: MiPoll | None, default=None
            Questionnaire to be created
        """

        return await super().create_quote(
            content=content,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            files=files,
            poll=poll,
            note_id=self._note_id,
        )

    @override
    async def translate(
        self,
        target_lang: str = "en-US",
    ) -> NoteTranslateResult:
        """Translate a note

        Endpoint: `/api/notes/translate`

        Parameters
        ----------
        target_lang : str, default='en'
            Target language

        Returns
        -------
        NoteTranslateResult
            Translated result
        """

        return await super().translate(target_lang=target_lang, note_id=self._note_id)

    @override
    async def un_renote(self) -> bool:
        """Releases the note renote for the specified Id

        Returns
        -------
        bool
            Whether the release was successful
        """

        return await super().un_renote(note_id=self._note_id)


class NoteActions(SharedNoteActions):
    def __init__(
        self,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        super().__init__(session=session, client=client)

    async def create(
        self,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance | None = None,
        no_extrace_mentions: bool = False,
        no_extract_hashtags: bool = False,
        no_extract_emojis: bool = False,
        reply_id: str | None = None,
        renote_id: str | None = None,
        channel_id: str | None = None,
        text: str | None = None,
        file_ids: list[MiFile | File | str] | None = None,
        media_ids: list[str] | None = None,
        poll: MiPoll | None = None,
    ) -> Note:
        data = create_note_body(
            text=text,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=not no_extrace_mentions,
            extract_hashtags=not no_extract_hashtags,
            extract_emojis=not no_extract_emojis,
            reply_id=reply_id,
            renote_id=renote_id,
            channel_id=channel_id,
            files=file_ids,
            media_ids=media_ids,
            poll=poll,
        )

        raw_created_note: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            auth=True,
            json=data,
        )

        return Note(raw_note=raw_created_note["created_note"], client=self._client)

    async def get_featured(
        self, limit: int = 10, until_id: str | None = None, channel_id: str | None = None
    ):
        """Get featured notes

        Endpoint: `/api/notes/featured`

        Parameters
        ----------
        limit : int, default=10
            limit
        until_id : str | None, default=None
            Until ID
        channel_id : str | None, default=None
            channel id

        Returns
        -------
        list[Note]
            featured notes
        """
        data = {"limit": limit, "untilId": until_id, "channelId": channel_id}
        res: list[INote] = await self._session.request(
            Route("POST", "/api/notes/featured"), json=data
        )
        return [Note(note, client=self._client) for note in res]

    async def get_global_time_line(
        self,
        with_files: bool = False,
        with_renotes: bool = True,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
    ):
        """Get global timeline

        Endpoint: `/api/notes/global-timeline`

        Parameters
        ----------
        with_files : bool, default=False
            Whether to include files
        with_renotes : bool, default=True
            Whether to include renote
        limit : int, default=10
            limit
        since_id : str | None, default=None
            Since ID
        until_id : str | None, default=None
            Until ID
        since_date : int | None, default=None
            Since date
        until_date : int | None, default=None
            Until date

        Returns
        -------
        list[Note]
            global timeline
        """
        data = {
            "withFiles": with_files,
            "withRenotes": with_renotes,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "sinceDate": since_date,
            "untilDate": until_date,
        }
        res: list[INote] = await self._session.request(
            Route("POST", "/api/notes/global-timeline"), json=data
        )
        return [Note(note, client=self._client) for note in res]

    async def get_hybird_time_line(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        allow_partial: bool = False,
        include_my_rnotes: bool = True,
        include_renoted_my_notes: bool = True,
        include_local_renotes: bool = True,
        with_files: bool = False,
        with_renotes: bool = True,
        with_replies: bool = False,
    ):
        data = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "allowPartial": allow_partial,
            "includeMyRenotes": include_my_rnotes,
            "includeRenotedMyNotes": include_renoted_my_notes,
            "includeLocalRenotes": include_local_renotes,
            "withFiles": with_files,
            "withRenotes": with_renotes,
            "withReplies": with_replies,
        }

        res = await self._session.request(
            Route("POST", "/api/notes/hybrid-timeline"), json=data, auth=True
        )

        return [Note(note, client=self._client) for note in res]

    async def get_local_timeline(
        self,
        with_files: bool = False,
        with_renotes: bool = True,
        with_replies: bool = False,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        allow_partial: bool = False,
        since_date: int | None = None,
        until_date: int | None = None,
    ):
        data = {
            "withFiles": with_files,
            "withRenotes": with_renotes,
            "withReplies": with_replies,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "allowPartial": allow_partial,
            "sinceDate": since_date,
            "untilDate": until_date,
        }

        res = await self._session.request(
            Route("POST", "/api/notes/local-timeline"), json=data, auth=True
        )

        return [Note(note, client=self._client) for note in res]

    async def get_mentions(
        self,
        following: bool = False,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        visibility: INoteVisibility = "public",
    ):
        """Get notes with mentions addressed to you

        Endpoint: `/api/notes/mentions`

        Parameters
        ----------
        following : bool, default=False
            Whether to include only users you follow
        limit : int, default=10
            limit
        since_id : str | None, default=None
            Since ID
        until_id : str | None, default=None
            Until ID
        visibility : INoteVisibility, default='public'
            Disclosure range
        """
        data = {
            "following": following,
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
            "visibility": visibility,
        }

        res: list[INote] = await self._session.request(
            Route("POST", "/api/notes/mentions"), json=data, auth=True
        )

        return [Note(note, client=self._client) for note in res]

    async def search_by_tag(
        self,
        tag: str,
        reply: bool | None = None,
        renote: bool | None = None,
        with_files: bool | None = None,
        poll: bool | None = None,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        query: list[list[str]] | None = None,
    ):
        """一致するタグのノートを取得します

        Endpoint: `/api/notes/search-by-tag`

        Returns
        -------
        list[Note]
            見つかったノート
        """
        data = {
            "tag": tag,
            "reply": reply,
            "renote": renote,
            "withFiles": with_files,
            "poll": poll,
            "sinceId": since_id,
            "untilId": until_id,
            "limit": limit,
            "query": query,
        }

        raw_notes: list[INote] = await self._session.request(
            Route("POST", "/api/notes/search-by-tag"), json=data, auth=True
        )
        return [Note(raw_note, client=self._client) for raw_note in raw_notes]

    async def get_all_search_by_tag(
        self,
        tag: str,
        reply: bool | None = None,
        renote: bool | None = None,
        with_files: bool | None = None,
        poll: bool | None = None,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        query: list[list[str]] | None = None,
    ) -> AsyncGenerator[Note, None]:
        """一致するタグのノートを取得します

        Endpoint: `/api/notes/search-by-tag`

        Returns
        -------
        AsyncGenerator[Note, None]
            見つかったノート
        """
        data = {
            "tag": tag,
            "reply": reply,
            "renote": renote,
            "withFiles": with_files,
            "poll": poll,
            "sinceId": since_id,
            "untilId": until_id,
            "limit": limit,
            "query": query,
        }

        pagination = Pagination[INote](
            http_client=self._session, route=Route("POST", "/api/notes/search-by-tag"), json=data
        )

        while pagination.is_final is False:
            raw_notes: list[INote] = await pagination.next()
            for raw_note in raw_notes:
                yield Note(raw_note, client=self._client)

    async def search(
        self,
        query: str,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        offset: int = 0,
        host: str | None = None,
        user_id: str | None = None,
        channel_id: str | None = None,
    ):  # それぞれのIDに合わせたメソッドをactionに実装する
        """ノートを検索します

        Endpoint: `/api/notes/search`

        Parameters
        ----------
        query : str
            検索クエリ
        since_id : str | None, default=None
            このIDより後のノートを取得します
        until_id : str | None, default=None
            このIDより前のノートを取得します
        limit : int, default=10
            取得するノートの数
        offset : int, default=0
            オフセット
        host : str | None, default=None
            対象のサーバー
            localhostは . で表現します
        user_id : str | None, default=None
            対象のユーザー
        channel_id : str | None, default=None
            対象のチャンネル

        Returns
        -------
        list[Note]
            検索結果
        """
        body = {
            "query": query,
            "sinceId": since_id,
            "untilId": until_id,
            "limit": limit,
            "offset": offset,
            "host": host,
            "userId": user_id,
            "channelId": channel_id,
        }

        raw_notes: list[INote] = await self._session.request(
            Route("POST", "/api/notes/search"), json=body
        )

        return [Note(raw_note, client=self._client) for raw_note in raw_notes]

    async def get_all_search(
        self,
        query: str,
        since_id: str | None = None,
        until_id: str | None = None,
        limit: int = 10,
        offset: int = 0,
        host: str | None = None,
        user_id: str | None = None,
        channel_id: str | None = None,
    ) -> AsyncGenerator[Note, None]:
        """ノートを検索します

        Endpoint: `/api/notes/search`

        Parameters
        ----------
        query : str
            検索クエリ
        since_id : str | None, default=None
            このIDより後のノートを取得します
        until_id : str | None, default=None
            このIDより前のノートを取得します
        limit : int, default=10
            取得するノートの数
        offset : int, default=0
            オフセット
        host : str | None, default=None
            対象のサーバー
            localhostは . で表現します
        user_id : str | None, default=None
            対象のユーザー
        channel_id : str | None, default=None
            対象のチャンネル

        Returns
        -------
        AsyncGenerator[Note, None]
            検索結果
        """
        body = {
            "query": query,
            "sinceId": since_id,
            "untilId": until_id,
            "limit": limit,
            "offset": offset,
            "host": host,
            "userId": user_id,
            "channelId": channel_id,
        }

        pagination = Pagination[INote](
            http_client=self._session, route=Route("POST", "/api/notes/search"), json=body
        )

        while pagination.is_final is False:
            raw_notes: list[INote] = await pagination.next()
            for raw_note in raw_notes:
                yield Note(raw_note, client=self._client)

    @deprecated
    async def send(
        self,
        text: str | None = None,
        visibility: INoteVisibility = "public",
        visible_user_ids: list[str] | None = None,
        cw: str | None = None,
        local_only: bool = False,
        reaction_acceptance: IReactionAcceptance = None,
        extract_mentions: bool = True,  # 元は noExtractMentions
        extract_hashtags: bool = True,  # 元は noExtractHashtags
        extract_emojis: bool = True,  # 元は noExtractEmojis
        reply_id: str | None = None,
        renote_id: str | None = None,
        channel_id: str | None = None,
        files: list[MiFile | File | str] | None = None,
        poll: MiPoll | None = None,
    ) -> Note:
        """Send a note

        Endpoint: `/api/notes/create`

        Parameters
        ----------
        text : str | None, default=None
            投稿する内容
        visibility : INoteVisibility, optional
            公開範囲, by default "public"
            Enum: "public" "home" "followers" "specified"
        visible_user_ids : list[str] | None, optional
            公開するユーザー, by default None
        cw : str | None, optional
            閲覧注意の文字, by default None
        local_only : bool, optional
            ローカルにのみ表示するか, by default False
        reaction_acceptance : IReactionAcceptance, optional
            リアクションの受け入れ設定, by default None
        extract_mentions : bool, optional
            メンションを展開するか, by default True
        extract_hashtags : bool, optional
            ハッシュタグを展開するか, by default True
        extract_emojis : bool, optional
            絵文字を展開するか, by default True
        reply_id : str | None, optional
            リプライ先のid, by default None
        renote_id : str | None, optional
            リノート先のid, by default None
        channel_id : str | None, optional
            チャンネルid, by default None
        files : list[MiFile | File | str], optional
            添付するファイルのリスト, by default None
        poll : MiPoll | None, optional
            アンケート, by default None

        Returns
        -------
        Note
            投稿したノート

        Raises
        ------
        ContentRequired
            [description]
        """
        body = create_note_body(
            text=text,
            visibility=visibility,
            visible_user_ids=visible_user_ids,
            cw=cw,
            local_only=local_only,
            reaction_acceptance=reaction_acceptance,
            extract_mentions=extract_mentions,
            extract_hashtags=extract_hashtags,
            extract_emojis=extract_emojis,
            reply_id=reply_id,
            renote_id=renote_id,
            channel_id=channel_id,
            files=files,
            poll=poll,
        )
        res: ICreatedNote = await self._session.request(
            Route("POST", "/api/notes/create"),
            json=body,
            auth=True,
            lower=True,
        )
        return Note(res["created_note"], client=self._client)

    @cache(group="get_note")
    async def get(self, note_id: str) -> Note:
        """Get a note

        Endpoint: `/api/notes/show`

        Parameters
        ----------
        note_id : str
            ノートのID

        Returns
        -------
        Note
            取得したノートID
        """
        raw_note: INote = await self._session.request(
            Route("POST", "/api/notes/show"),
            json={"noteId": note_id},
            auth=True,
            lower=True,
        )
        return Note(raw_note=raw_note, client=self._client)

    @cache(group="get_note", override=True)
    async def fetch(self, note_id: str) -> Note:
        """Get a note.

        update the cache of the :py:meth:`mipac.actions.note.NoteActions.get` method

        Endpoint: `/api/notes/show`

        Parameters
        ----------
        note_id : str
            note id

        Returns
        -------
        Note
            note
        """
        raw_note: INote = await self._session.request(
            Route("POST", "/api/notes/show"),
            json={"noteId": note_id},
            auth=True,
            lower=True,
        )
        return Note(raw_note=raw_note, client=self._client)

    async def gets(
        self,
        local: bool = False,
        reply: bool = False,
        renote: bool = False,
        with_files: bool = False,
        poll: bool = False,
        limit: int = 100,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        get_all: bool = False,
    ) -> AsyncGenerator[Note, None]:
        if limit > 100:
            raise ValueError("limit は100以下である必要があります")

        if get_all:
            limit = 100

        body = remove_dict_empty(
            {
                "local": local,
                "reply": reply,
                "renote": renote,
                "withFiles": with_files,
                "poll": poll,
                "limit": limit,
                "sinceId": since_id,
                "untilId": until_id,
            }
        )

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/notes"), json=body, limit=limit
        )

        while pagination.is_final is False:
            raw_notes = await pagination.next()
            for raw_note in raw_notes:
                yield Note(raw_note=raw_note, client=self._client)

    async def get_time_line(
        self,
        list_id: str,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        include_renote_my_notes: bool = True,
        include_local_renotes: bool = True,
        with_renotes: bool = True,
        with_files: bool = True,
    ) -> list[Note]:
        """
        Get the timeline of the specified list

        Endpoint: `/api/notes/user-list-timeline`

        Parameters
        ----------
        list_id : str
            List ID
        limit : int, default=10
            limit
        since_id : str | None, default=None
            Since ID
        until_id : str | None, default=None
            Until ID
        since_date : int | None, default=None
            Since date
        until_date : int | None, default=None
            Until date
        include_renote_my_notes : bool, default=True
            Whether to include your own renote
        include_local_renotes : bool, default=True
            Whether to include local renote
        with_renotes : bool, default=True
            Whether to include renote
        with_files : bool, default=True
            Whether to include files

        Returns
        -------
        list[Note]

        Notes
        -----
        """
        data = {
            "limit": limit,
            "listId": list_id,
            "includeRenoteMyNotes": include_renote_my_notes,
            "includeLocalRenotes": include_local_renotes,
            "withRenotes": with_renotes,
            "withFiles": with_files,
            "sinceId": since_id,
            "untilId": until_id,
            "sinceDate": since_date,
            "untilDate": until_date,
        }

        raw_notes: list[INote] = await self._session.request(
            Route("POST", "/api/notes/user-list-timeline"), json=data, auth=True
        )

        return [Note(raw_note, client=self._client) for raw_note in raw_notes]

    async def get_all_time_line(
        self,
        list_id: str,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        since_date: int | None = None,
        until_date: int | None = None,
        include_renote_my_notes: bool = True,
        include_local_renotes: bool = True,
        with_renotes: bool = True,
        with_files: bool = True,
    ):
        data = {
            "limit": limit,
            "listId": list_id,
            "includeRenoteMyNotes": include_renote_my_notes,
            "includeLocalRenotes": include_local_renotes,
            "withRenotes": with_renotes,
            "withFiles": with_files,
            "sinceId": since_id,
            "untilId": until_id,
            "sinceDate": since_date,
            "untilDate": until_date,
        }

        pagination = Pagination[INote](
            self._session, Route("POST", "/api/notes/user-list-timeline"), json=data, auth=True
        )

        while pagination.is_final is False:
            raw_notes: list[INote] = await pagination.next()
            for raw_note in raw_notes:
                yield Note(raw_note, client=self._client)



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.common import ID
from mipac.models.emoji import EmojiDetailed
from mipac.types.common import IID
from mipac.types.emoji import IEmojiDetailed
from mipac.utils.format import remove_dict_missing
from mipac.utils.pagination import Pagination
from mipac.utils.util import MISSING

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class SharedAdminEmojiActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client

    async def copy(self, *, emoji_id: str):
        """Copy an emoji

        Parameters
        ----------
        emoji_id : str
            The id of the emoji
        """
        raw_id: IID = await self.__session.request(
            Route("POST", "/api/admin/emoji/copy"), json={"emojiId": emoji_id}
        )

        return ID(raw_id=raw_id)

    async def delete(self, *, emoji_id: str) -> bool:
        """Delete an emoji

        Parameters
        ----------
        id : str
            The id of the emoji

        Returns
        -------
        bool
            Whether the operation was successful
        """
        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/delete"), json={"id": emoji_id}
        )

        return res

    async def update(
        self,
        name: str = MISSING,
        file_id: str = MISSING,
        category: str | None = MISSING,
        aliases: list[str] | None = MISSING,
        is_sensitive: bool = MISSING,
        local_only: bool = MISSING,
        role_ids_that_can_be_used_this_emoji_as_reaction: list[str] = MISSING,
        *,
        emoji_id: str,
    ) -> bool:
        """Update an emoji

        Parameters
        ----------
        name : str
            The name of the emoji
        file_id : str
            The id of the file
        category : str, optional
            The category of the emoji, by default None
        aliases : list[str], optional
            The aliases of the emoji, by default None
        is_sensitive : bool
            Whether the emoji is sensitive
        local_only : bool
            Whether the emoji is local only
        role_ids_that_can_be_used_this_emoji_as_reaction : list[str]
            The role ids that can be used this emoji as reaction
        emoji_id : str
            The id of the emoji

        Returns
        -------
        bool
            Whether the operation was successful
        """

        body = remove_dict_missing(
            {
                "id": emoji_id,
                "name": name,
                "fileId": file_id,
                "category": category,
                "aliases": aliases,
                "isSensitive": is_sensitive,
                "localOnly": local_only,
                "roleIdsThatCanBeUsedThisEmojiAsReaction": role_ids_that_can_be_used_this_emoji_as_reaction,
            }
        )

        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/update"), json=body
        )
        return res


class ClientAdminEmojiActions(SharedAdminEmojiActions):
    def __init__(self, *, emoji_id: str, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)
        self.__emoji_id: str = emoji_id

    @override
    async def copy(self) -> ID:
        return await super().copy(emoji_id=self.__emoji_id)

    @override
    async def delete(self) -> bool:
        return await super().delete(emoji_id=self.__emoji_id)

    @override
    async def update(
        self,
        name: str = MISSING,
        file_id: str = MISSING,
        category: str | None = MISSING,
        aliases: list[str] | None = MISSING,
        is_sensitive: bool = MISSING,
        local_only: bool = MISSING,
        role_ids_that_can_be_used_this_emoji_as_reaction: list[str] = MISSING,
    ) -> bool:
        return await super().update(
            name=name,
            file_id=file_id,
            category=category,
            aliases=aliases,
            is_sensitive=is_sensitive,
            local_only=local_only,
            role_ids_that_can_be_used_this_emoji_as_reaction=role_ids_that_can_be_used_this_emoji_as_reaction,
            emoji_id=self.__emoji_id,
        )


class AdminEmojiActions(SharedAdminEmojiActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def add_aliases_bulk(self, ids: list[str], aliases: list[str]) -> bool:
        """Add aliases to emojis in bulk

        Parameters
        ----------
        ids : list[str]
            The ids of the emojis
        aliases : list[str]
            The aliases to add

        Returns
        -------
        bool
            Whether the operation was successful
        """
        body = {"ids": ids, "aliases": aliases}

        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/add-aliases-bulk"), json=body
        )

        return res

    async def add(
        self,
        name: str,
        file_id: str,
        category: str | None = None,
        aliases: list[str] | None = None,
        license: str | None = None,
        is_sensitive: bool = MISSING,
        local_only: bool = MISSING,
    ) -> EmojiDetailed:
        """Add a new emoji

        Parameters
        ----------
        name : str
            The name of the emoji
        file_id : str
            The id of the file
        category : str, optional
            The category of the emoji, by default None
        aliases : list[str], optional
            The aliases of the emoji, by default None
        license : str, optional
            The license of the emoji, by default None
        is_sensitive : bool
            Whether the emoji is sensitive
        local_only : bool
            Whether the emoji is local only

        Returns
        -------
        EmojiDetailed
            The added emoji
        """

        body = remove_dict_missing(
            {
                "name": name,
                "fileId": file_id,
                "category": category,
                "aliases": aliases,
                "license": license,
                "isSensitive": is_sensitive,
                "localOnly": local_only,
            }
        )

        raw_emoji: IEmojiDetailed = await self.__session.request(
            Route("POST", "/api/admin/emoji/add"), json=body
        )

        return EmojiDetailed(raw_emoji, client=self.__client)

    async def delete_bulk(self, ids: list[str]) -> bool:
        """Delete emojis in bulk

        Parameters
        ----------
        ids : list[str]
            The ids of the emojis

        Returns
        -------
        bool
            Whether the operation was successful
        """
        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/delete-bulk"), json={"ids": ids}
        )

        return res

    async def get_list_remote(
        self,
        query: str | None = None,
        host: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> list[EmojiDetailed]:
        """List remote emojis

        Parameters
        ----------
        query : str, optional
            The query to search for, by default None
        host : str, optional
            The host of the emoji, by default None
        limit : int, optional
            The limit of the emojis to get, by default 10
        since_id : str, optional
            The id to get emojis since, by default None
        until_id : str, optional
            The id to get emojis until, by default None

        Returns
        -------
        list[EmojiDetailed]
            The emojis
        """
        body = remove_dict_missing(
            {
                "query": query,
                "host": host,
                "limit": limit,
                "sinceId": since_id,
                "untilId": until_id,
            }
        )

        raw_emojis: list[IEmojiDetailed] = await self.__session.request(
            Route("POST", "/api/admin/emoji/list-remote"), json=body
        )

        return [EmojiDetailed(raw_emoji, client=self.__client) for raw_emoji in raw_emojis]

    async def get_all_list_remote(
        self,
        query: str | None = None,
        host: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> AsyncGenerator[EmojiDetailed, None]:
        """List remote emojis

        Parameters
        ----------
        query : str, optional
            The query to search for, by default None
        host : str, optional
            The host of the emoji, by default None
        limit : int, optional
            The limit of the emojis to get, by default 10
        since_id : str, optional
            The id to get emojis since, by default None
        until_id : str, optional
            The id to get emojis until, by default None

        Yields
        ------
        AsyncGenerator[EmojiDetailed, None]
            The emojis
        """
        body = remove_dict_missing(
            {
                "query": query,
                "host": host,
                "limit": limit,
                "sinceId": since_id,
                "untilId": until_id,
            }
        )

        pagination = Pagination[IEmojiDetailed](
            self.__session, Route("POST", "/api/admin/emoji/list-remote"), body
        )

        while pagination.is_final is False:
            for raw_emoji in await pagination.next():
                yield EmojiDetailed(raw_emoji, client=self.__client)

    async def get_list(
        self,
        query: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> list[EmojiDetailed]:
        """List emojis

        Parameters
        ----------
        query : str, optional
            The query to search for, by default None
        limit : int, optional
            The limit of the emojis to get, by default 10
        since_id : str, optional
            The id to get emojis since, by default None
        until_id : str, optional
            The id to get emojis until, by default None

        Returns
        -------
        list[EmojiDetailed]
            The emojis
        """
        body = remove_dict_missing(
            {"query": query, "limit": limit, "sinceId": since_id, "untilId": until_id}
        )

        raw_emojis: list[IEmojiDetailed] = await self.__session.request(
            Route("POST", "/api/admin/emoji/list"), json=body
        )

        return [EmojiDetailed(raw_emoji, client=self.__client) for raw_emoji in raw_emojis]

    async def get_all_list(
        self,
        query: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> AsyncGenerator[EmojiDetailed, None]:
        """List all emojis

        Parameters
        ----------
        query : str, optional
            The query to search for, by default None
        limit : int, optional
            The limit of the emojis to get, by default 10
        since_id : str, optional
            The id to get emojis since, by default None
        until_id : str, optional
            The id to get emojis until, by default None

        Yields
        ------
        AsyncGenerator[EmojiDetailed, None]
            The emojis
        """

        body = remove_dict_missing(
            {"query": query, "limit": limit, "sinceId": since_id, "untilId": until_id}
        )

        pagination = Pagination[IEmojiDetailed](
            self.__session, Route("POST", "/api/admin/emoji/list"), body
        )

        while pagination.is_final is False:
            for raw_emoji in await pagination.next():
                yield EmojiDetailed(raw_emoji, client=self.__client)

    async def remove_aliases_bulk(self, ids: list[str], aliases: list[str]) -> bool:
        """Remove aliases from emojis in bulk

        Parameters
        ----------
        ids : list[str]
            The ids of the emojis
        aliases : list[str]
            The aliases to remove

        Returns
        -------
        bool
            Whether the operation was successful
        """
        body = {"ids": ids, "aliases": aliases}

        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/remove-aliases-bulk"), json=body
        )

        return res

    async def set_aliases_bulk(self, ids: list[str], aliases: list[str]) -> bool:
        """Set aliases to emojis in bulk

        Parameters
        ----------
        ids : list[str]
            The ids of the emojis
        aliases : list[str]
            The aliases to set

        Returns
        -------
        bool
            Whether the operation was successful
        """
        body = {"ids": ids, "aliases": aliases}

        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/set-aliases-bulk"), json=body
        )

        return res

    async def set_category_bulk(self, ids: list[str], category: str) -> bool:
        """Set category to emojis in bulk

        Parameters
        ----------
        ids : list[str]
            The ids of the emojis
        category : str
            The category to set

        Returns
        -------
        bool
            Whether the operation was successful
        """
        body = {"ids": ids, "category": category}

        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/set-category-bulk"), json=body
        )

        return res

    async def set_license_bulk(self, ids: list[str], license: str) -> bool:
        """Set license to emojis in bulk

        Parameters
        ----------
        ids : list[str]
            The ids of the emojis
        license : str
            The license to set

        Returns
        -------
        bool
            Whether the operation was successful
        """
        body = {"ids": ids, "license": license}

        res: bool = await self.__session.request(
            Route("POST", "/api/admin/emoji/set-license-bulk"), json=body
        )

        return res



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.blocking import BlockingActions, ClientBlockingActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class ClientBlockingManager(BlockingActions):
    def __init__(self, user_id: str, *, session: HTTPClient, client: ClientManager):
        self.__action: ClientBlockingActions = ClientBlockingActions(
            user_id=user_id, session=session, client=client
        )

    @property
    def action(self) -> ClientBlockingActions:
        return self.__action


class BlockingManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__action: BlockingActions = BlockingActions(session=session, client=client)

    @property
    def action(self) -> BlockingActions:
        return self.__action



from __future__ import annotations

from typing import TYPE_CHECKING, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.utils.util import deprecated

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class SharedFavoriteActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def create(self, *, note_id: str) -> bool:
        data = {"noteId": note_id}
        return bool(
            await self._session.request(
                Route("POST", "/api/notes/favorites/create"),
                json=data,
                auth=True,
            )
        )

    @deprecated
    async def add(self, *, note_id: str) -> bool:
        return await self.create(note_id=note_id)

    async def delete(self, *, note_id: str) -> bool:
        data = {"noteId": note_id}
        return bool(
            await self._session.request(
                Route("POST", "/api/notes/favorites/delete"),
                json=data,
                auth=True,
            )
        )

    @deprecated
    async def remove(self, *, note_id: str) -> bool:
        return await self.delete(note_id=note_id)


class ClientFavoriteActions(SharedFavoriteActions):
    def __init__(self, note_id: str, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)
        self.__note_id: str = note_id

    @override
    async def create(self) -> bool:
        return await super().create(note_id=self.__note_id)

    @deprecated
    @override
    async def add(self) -> bool:
        return await super().create(note_id=self.__note_id)

    @override
    async def delete(self) -> bool:
        return await super().delete(note_id=self.__note_id)

    @deprecated
    @override
    async def remove(self) -> bool:
        return await super().delete(note_id=self.__note_id)


class FavoriteActions(SharedFavoriteActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from mipac.models.lite.meta import PartialMeta
from mipac.types.meta import (
    IAdminMeta,
    IFeatures,
    IMeta,
    ISensitiveMediaDetection,
    ISensitiveMediaDetectionSentivity,
)

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class Features:
    def __init__(self, raw_features: IFeatures) -> None:
        self.__raw_features = raw_features

    @property
    def registration(self) -> bool:
        return self.__raw_features["registration"]

    @property
    def email_required_for_signup(self) -> bool:
        return self.__raw_features["email_required_for_signup"]

    @property
    def hcaptcha(self) -> bool:
        return self.__raw_features["hcaptcha"]

    @property
    def recaptcha(self) -> bool:
        return self.__raw_features["recaptcha"]

    @property
    def turnstile(self) -> bool:
        return self.__raw_features["turnstile"]

    @property
    def object_storage(self) -> bool:
        return self.__raw_features["object_storage"]

    @property
    def service_worker(self) -> bool:
        return self.__raw_features["service_worker"]

    @property
    def miauth(self) -> bool:
        return self.__raw_features["miauth"]

    def _get(self, key: str) -> Any | None:
        return self.__raw_features.get(key)


class Meta(PartialMeta[IMeta]):
    def __init__(self, instance_metadata: IMeta, *, client: ClientManager) -> None:
        super().__init__(instance_metadata, client=client)

    @property
    def features(self) -> Features:
        return Features(self._raw_meta["features"])

    @property
    def cache_remote_files(self) -> bool:
        return self._raw_meta["cache_remote_files"]

    @property
    def cache_remote_sensitive_files(self) -> bool:
        return self._raw_meta["cache_remote_sensitive_files"]

    @property
    def require_setup(self) -> bool:
        return self._raw_meta["require_setup"]

    @property
    def proxy_account_name(self) -> str:
        return self._raw_meta["proxy_account_name"]


class AdminMeta:
    def __init__(self, raw_meta: IAdminMeta, *, client: ClientManager) -> None:
        self.__raw_meta: IAdminMeta = raw_meta
        self.__client: ClientManager = client

    @property
    def maintainer_name(self) -> str | None:
        return self.__raw_meta["maintainer_name"]

    @property
    def maintainer_email(self) -> str | None:
        return self.__raw_meta["maintainer_email"]

    @property
    def version(self) -> str:
        return self.__raw_meta["version"]

    @property
    def name(self) -> str | None:
        return self.__raw_meta["name"]

    @property
    def short_name(self) -> str | None:
        return self.__raw_meta["short_name"]

    @property
    def uri(self) -> str:
        return self.__raw_meta["uri"]

    @property
    def description(self) -> str | None:
        return self.__raw_meta["description"]

    @property
    def langs(self) -> list[str]:
        return self.__raw_meta["langs"]

    @property
    def tos_url(self) -> str:
        return self.__raw_meta["tos_url"]

    @property
    def repository_url(self) -> str:
        return self.__raw_meta["repository_url"]

    @property
    def feedback_url(self) -> str:
        return self.__raw_meta["feedback_url"]

    @property
    def disable_registration(self) -> bool:
        return self.__raw_meta["disable_registration"]

    @property
    def email_required_for_signup(self) -> bool:
        return self.__raw_meta["email_required_for_signup"]

    @property
    def enable_hcaptcha(self) -> bool:
        return self.__raw_meta["enable_hcaptcha"]

    @property
    def hcaptcha_site_key(self) -> str | None:
        return self.__raw_meta["hcaptcha_site_key"]

    @property
    def enable_recaptcha(self) -> bool:
        return self.__raw_meta["enable_recaptcha"]

    @property
    def recaptcha_site_key(self) -> str:
        return self.__raw_meta["recaptcha_site_key"]

    @property
    def enable_turnstile(self) -> bool:
        return self.__raw_meta["enable_turnstile"]

    @property
    def turnstile_site_key(self) -> str:
        return self.__raw_meta["turnstile_site_key"]

    @property
    def sw_publickey(self) -> str | None:
        return self.__raw_meta["sw_publickey"]

    @property
    def theme_color(self) -> str:
        return self.__raw_meta["theme_color"]

    @property
    def mascot_image_url(self) -> str:
        return self.__raw_meta["mascot_image_url"]

    @property
    def banner_url(self) -> str | None:
        return self.__raw_meta["banner_url"]

    @property
    def server_error_image_url(self) -> str | None:
        return self.__raw_meta["server_error_image_url"]

    @property
    def not_found_image_url(self) -> str | None:
        return self.__raw_meta["not_found_image_url"]

    @property
    def info_image_url(self) -> str | None:
        return self.__raw_meta["info_image_url"]

    @property
    def icon_url(self) -> str | None:
        return self.__raw_meta["icon_url"]

    @property
    def appint_icon_url(self) -> str | None:
        return self.__raw_meta["appint_icon_url"]

    @property
    def background_image_url(self) -> str | None:
        return self.__raw_meta["background_image_url"]

    @property
    def logo_image_url(self) -> str | None:
        return self.__raw_meta["logo_image_url"]

    @property
    def default_light_theme(self) -> str | None:
        return self.__raw_meta["default_light_theme"]

    @property
    def default_dark_theme(self) -> str | None:
        return self.__raw_meta["default_dark_theme"]

    @property
    def enable_email(self) -> bool:
        return self.__raw_meta["enable_email"]

    @property
    def enable_service_worker(self) -> bool:
        return self.__raw_meta["enable_service_worker"]

    @property
    def translator_available(self) -> bool:
        return self.__raw_meta["translator_available"]

    @property
    def cache_remote_files(self) -> bool:
        return self.__raw_meta["cache_remote_files"]

    @property
    def cache_remote_sensitive_files(self) -> bool:
        return self.__raw_meta["cache_remote_sensitive_files"]

    @property
    def pinned_users(self) -> list[str]:
        return self.__raw_meta["pinned_users"]

    @property
    def hidden_tags(self) -> list[str]:
        return self.__raw_meta["hidden_tags"]

    @property
    def blocked_hosts(self) -> list[str]:
        return self.__raw_meta["blocked_hosts"]

    @property
    def sensitive_words(self) -> list[str]:
        return self.__raw_meta["sensitive_words"]

    @property
    def preserved_usernames(self) -> list[str]:
        return self.__raw_meta["preserved_usernames"]

    @property
    def hcaptcha_secret_key(self) -> str | None:
        return self.__raw_meta["hcaptcha_secret_key"]

    @property
    def recaptcha_secret_key(self) -> str:
        return self.__raw_meta["recaptcha_secret_key"]

    @property
    def turnstile_secret_key(self) -> str:
        return self.__raw_meta["turnstile_secret_key"]

    @property
    def sensitive_media_detection(self) -> ISensitiveMediaDetection:
        return self.__raw_meta["sensitive_media_detection"]

    @property
    def sensitive_media_detection_sensitivity(self) -> ISensitiveMediaDetectionSentivity:
        return self.__raw_meta["sensitive_media_detection_sensitivity"]

    @property
    def set_sensitive_flag_automatically(self) -> bool:
        return self.__raw_meta["set_sensitive_flag_automatically"]

    @property
    def enable_sensitive_media_detection_for_videos(self) -> bool:
        return self.__raw_meta["enable_sensitive_media_detection_for_videos"]

    @property
    def proxy_account_id(self) -> str:
        return self.__raw_meta["proxy_account_id"]

    @property
    def summaly_proxy(self) -> str | None:
        return self.__raw_meta["summaly_proxy"]

    @property
    def email(self) -> str:
        return self.__raw_meta["email"]

    @property
    def smtp_secure(self) -> bool:
        return self.__raw_meta["smtp_secure"]

    @property
    def smtp_host(self) -> str:
        return self.__raw_meta["smtp_host"]

    @property
    def smtp_port(self) -> int:
        return self.__raw_meta["smtp_port"]

    @property
    def smtp_user(self) -> str:
        return self.__raw_meta["smtp_user"]

    @property
    def smtp_pass(self) -> str:
        return self.__raw_meta["smtp_pass"]

    @property
    def sw_private_key(self) -> str | None:
        return self.__raw_meta["sw_private_key"]

    @property
    def use_object_storage(self) -> bool:
        return self.__raw_meta["use_object_storage"]

    @property
    def object_storage_base_url(self) -> str:
        return self.__raw_meta["object_storage_base_url"]

    @property
    def object_storage_bucket(self) -> str:
        return self.__raw_meta["object_storage_bucket"]

    @property
    def object_storage_prefix(self) -> str:
        return self.__raw_meta["object_storage_prefix"]

    @property
    def object_storage_endpoint(self) -> str:
        return self.__raw_meta["object_storage_endpoint"]

    def _get(self, key: str) -> Any | None:
        return self.__raw_meta.get(key)



from __future__ import annotations

from typing import TYPE_CHECKING, Literal, NotRequired, TypedDict

if TYPE_CHECKING:
    from mipac.models.lite.user import PartialUser


__all__ = ("IFileProperties", "IFolder", "IFile", "IDriveSort", "IDriveStatus")

IDriveSort = Literal["+createdAt", "-createdAt", "+name", "-name", "+size", "-size"]


class IDriveStatus(TypedDict):
    capacity: int
    usage: int


class IFileProperties(TypedDict):
    """プロパティー情報"""

    width: NotRequired[int]
    height: NotRequired[int]
    orientation: NotRequired[int]
    avg_color: NotRequired[str]


class IFolder(TypedDict):
    """フォルダーの情報"""

    id: str
    created_at: str
    name: str
    parent_id: str | None
    folders_count: NotRequired[int]
    files_count: NotRequired[int]
    parent: NotRequired[IFolder]


class IFile(TypedDict):
    """ファイル情報"""

    id: str
    created_at: str
    name: str
    type: str
    md5: str
    size: int
    is_sensitive: bool
    blurhash: str | None
    properties: IFileProperties
    url: str
    thumbnail_url: str | None
    comment: str | None
    folder_id: str | None
    folder: NotRequired[IFolder | None]
    user_id: str | None
    user: NotRequired[PartialUser | None]



from __future__ import annotations

from typing import TYPE_CHECKING, Any, TypeVar

from mipac.types.ads import IAdPlaces, IPartialAd

if TYPE_CHECKING:
    from mipac.manager.admins.ad import ClientAdminAdManager
    from mipac.manager.client import ClientManager


T = TypeVar("T", bound=IPartialAd)


class PartialAd[T: IPartialAd]:
    def __init__(self, raw_ad: T, *, client: ClientManager) -> None:
        self._raw_ad: T = raw_ad
        self._client = client

    @property
    def id(self) -> str:
        """広告のID

        Returns
        -------
        str
            広告のID
        """
        return self._raw_ad["id"]

    @property
    def url(self) -> str:
        """広告のURL

        Returns
        -------
        str
            広告のURL
        """
        return self._raw_ad["url"]

    @property
    def place(self) -> IAdPlaces:
        return self._raw_ad["place"]

    @property
    def ratio(self) -> int:
        return self._raw_ad["ratio"]

    @property
    def image_url(self) -> str:
        return self._raw_ad["image_url"]

    @property
    def day_of_week(self) -> int:
        return self._raw_ad["day_of_week"]

    @property
    def api(self) -> ClientAdminAdManager:
        return self._client.admin._create_client_ad_manager(ad_id=self.id)

    def _get(self, key: str) -> Any | None:
        """You can access the raw response data directly by specifying the key

        Returns
        -------
        Any | None
            raw response data
        """
        return self._raw_ad.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, PartialAd) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Literal

from mipac.models.lite.ad import PartialAd
from mipac.types.ads import IAd
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class Ad(PartialAd[IAd]):
    def __init__(self, ad_data: IAd, *, client: ClientManager) -> None:
        super().__init__(ad_data, client=client)

    @property
    def expires_at(self) -> datetime:
        """いつ広告が終了するか

        Returns
        -------
        datetime
            いつ広告が終了するか
        """
        return str_to_datetime(self._raw_ad["expires_at"])

    @property
    def starts_at(self) -> datetime:
        """いつ広告が開始するか

        Returns
        -------
        datetime
            いつ広告が開始するか
        """
        return str_to_datetime(self._raw_ad["starts_at"])

    @property
    def priority(self) -> Literal["high" "middle" "low"]:
        """広告の優先度

        Returns
        -------
        Literal["high" "middle" "low"]
            広告の優先度
        """
        return self._raw_ad["priority"]

    @property
    def memo(self) -> str | None:
        """広告のメモ

        Returns
        -------
        str | None
            広告のメモ
        """
        return self._raw_ad["memo"]



from __future__ import annotations

from typing import TYPE_CHECKING, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.emoji import CustomEmoji
from mipac.models.note import NoteReaction
from mipac.types.meta import IPartialMeta
from mipac.types.note import INoteReaction
from mipac.utils.cache import cache
from mipac.utils.format import remove_dict_empty

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class SharedReactionActions(AbstractAction):
    def __init__(self, *, session: HTTPClient, client: ClientManager) -> None:
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def add(self, reaction: str, *, note_id: str) -> bool:
        """Add reaction to note

        Endpoint: `/api/notes/reactions/create`

        Parameters
        ----------
        reaction : str
            reaction
        note_id : str, optional
            note id, by default None

        Returns
        -------
        bool
            success or not
        """
        data = remove_dict_empty({"noteId": note_id, "reaction": reaction})
        route = Route("POST", "/api/notes/reactions/create")
        res: bool = await self._session.request(route, json=data, auth=True, lower=True)
        return bool(res)

    async def remove(self, *, note_id: str) -> bool:
        """Remove reaction from note

        Endpoint: `/api/notes/reactions/delete`

        Parameters
        ----------
        note_id : str, optional
            note id, by default None

        Returns
        -------
        bool
            success or not
        """
        data = remove_dict_empty({"noteId": note_id})
        route = Route("POST", "/api/notes/reactions/delete")
        res: bool = await self._session.request(route, json=data, auth=True, lower=True)
        return bool(res)

    @cache(group="get_note_reaction")
    async def get_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        note_id: str,
    ) -> list[NoteReaction]:
        data = remove_dict_empty(
            {
                "noteId": note_id,
                "limit": limit,
                "type": type,
                "sinceId": since_id,
                "untilId": until_id,
            }
        )
        res: list[INoteReaction] = await self._session.request(
            Route("POST", "/api/notes/reactions"),
            json=data,
            auth=True,
            lower=True,
        )
        return [NoteReaction(i, client=self._client) for i in res]

    @cache(group="get_note_reaction", override=True)
    async def fetch_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        *,
        note_id: str,
    ) -> list[NoteReaction]:
        return await self.get_reactions(
            type=type, note_id=note_id, limit=limit, since_id=since_id, until_id=until_id
        )


class ClientReactionActions(SharedReactionActions):
    def __init__(self, note_id: str, *, session: HTTPClient, client: ClientManager) -> None:
        super().__init__(session=session, client=client)
        self.__note_id: str = note_id

    @override
    async def add(self, reaction: str) -> bool:
        """Add reaction to note

        Endpoint: `/api/notes/reactions/create`

        Parameters
        ----------
        reaction : str
            reaction

        Returns
        -------
        bool
            success or not
        """
        return await super().add(reaction=reaction, note_id=self.__note_id)

    @override
    async def remove(self) -> bool:
        """Remove reaction from note

        Endpoint: `/api/notes/reactions/delete`

        Returns
        -------
        bool
            success or not
        """
        return await super().remove(note_id=self.__note_id)

    @override
    async def get_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> list[NoteReaction]:
        return await super().get_reactions(
            type=type, note_id=self.__note_id, limit=limit, since_id=since_id, until_id=until_id
        )

    @override
    async def fetch_reactions(
        self,
        type: str | None = None,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
    ) -> list[NoteReaction]:
        return await super().fetch_reactions(
            type=type, note_id=self.__note_id, limit=limit, since_id=since_id, until_id=until_id
        )


class ReactionActions(SharedReactionActions):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        super().__init__(session=session, client=client)

    async def get_emoji_list(
        self,
    ) -> list[CustomEmoji]:  # TODO: metaからemojisは削除されてるので別の方法に切り替える
        data: IPartialMeta = await self._session.request(
            Route("GET", "/api/meta"),
            json={"detail": False},
            auth=True,
            replace_list={"ToSUrl": "tos_url", "ToSTextUrl": "tos_text_url"},
        )
        return [CustomEmoji(i, client=self._client) for i in data.get("emojis", [])]



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from mipac.models.lite.emoji import PartialCustomEmoji
from mipac.types.emoji import ICustomEmoji, IEmojiDetailed, IEmojiSimple

if TYPE_CHECKING:
    from mipac.client import ClientManager
    from mipac.manager.admins.emoji import ClientAdminEmojiManager


__all__ = ("CustomEmoji",)


class CustomEmoji(PartialCustomEmoji):
    def __init__(self, emoji: ICustomEmoji, *, client: ClientManager):
        super().__init__(emoji, client=client)
        self.__emoji: ICustomEmoji = emoji

    @property
    def id(self) -> str:
        return self.__emoji["id"]

    @property
    def aliases(self) -> list[str]:
        return self.__emoji["aliases"]

    @property
    def category(self) -> str:
        return self.__emoji["category"]

    @property
    def license(self) -> str | None:
        return self.__emoji["license"]

    @property
    def host(self) -> str | None:
        return self.__emoji["host"]

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, CustomEmoji) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class EmojiSimple:
    """Misskeyの `EmojiSimple` を表すクラス"""

    def __init__(self, raw_emoji_simple: IEmojiSimple, *, client: ClientManager):
        self.__raw_emoji_simple: IEmojiSimple = raw_emoji_simple
        self.__client: ClientManager = client

    @property
    def aliaces(self) -> list[str]:
        return self.__raw_emoji_simple["aliaces"]

    @property
    def name(self) -> str:
        return self.__raw_emoji_simple["name"]

    @property
    def category(self) -> str | None:
        return self.__raw_emoji_simple["category"]

    @property
    def url(self) -> str:
        return self.__raw_emoji_simple["url"]

    @property
    def local_only(self) -> bool:
        return self.__raw_emoji_simple["local_only"]

    @property
    def is_sensitive(self) -> bool:
        return self.__raw_emoji_simple.get("is_sensitive", False)

    @property
    def role_ids_that_can_be_used_this_emoji_as_reaction(self) -> list[str]:
        return self.__raw_emoji_simple.get("role_ids_that_can_be_used_this_emoji_as_reaction", [])

    def _get(self, key: str) -> Any | None:
        return self.__raw_emoji_simple.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, EmojiSimple) and self.name == __value.name

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)


class EmojiDetailed:
    def __init__(self, raw_emoji_detailed: IEmojiDetailed, *, client: ClientManager):
        self.__raw_emoji_detailed: IEmojiDetailed = raw_emoji_detailed
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__raw_emoji_detailed["id"]

    @property
    def aliaces(self) -> list[str]:
        return self.__raw_emoji_detailed["aliaces"]

    @property
    def name(self) -> str:
        return self.__raw_emoji_detailed["name"]

    @property
    def category(self) -> str | None:
        return self.__raw_emoji_detailed["category"]

    @property
    def host(self) -> str | None:
        return self.__raw_emoji_detailed["host"]

    @property
    def url(self) -> str:
        return self.__raw_emoji_detailed["url"]

    @property
    def license(self) -> str | None:
        return self.__raw_emoji_detailed["license"]

    @property
    def is_sensitive(self) -> bool:
        return self.__raw_emoji_detailed["is_sensitive"]

    @property
    def local_only(self) -> bool:
        return self.__raw_emoji_detailed["local_only"]

    @property
    def role_ids_that_can_be_used_this_emoji_as_reaction(self) -> list[str]:
        return self.__raw_emoji_detailed["role_ids_that_can_be_used_this_emoji_as_reaction"]

    @property
    def api(self) -> ClientAdminEmojiManager:
        return self.__client.admin._create_client_admin_emoji_manager(self.id)

    def _get(self, key: str) -> Any | None:
        return self.__raw_emoji_detailed.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, EmojiDetailed) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from typing import TypedDict

from mipac.types.user import IPartialUser


class IInviteCode(TypedDict):
    id: str
    code: str
    expires_at: str | None
    created_at: str
    created_by: IPartialUser | None
    used_by: IPartialUser | None
    used_at: str | None
    used: bool


class IInviteLimit(TypedDict):
    remaining: int | None



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from mipac.models.lite.instance import LiteInstance
from mipac.types.user import IAvatarDecoration, IBadgeRole, IPartialUser, IUserOnlineStatus
from mipac.utils.util import deprecated

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.manager.user import ClientUserManager


class BadgeRole[T: IBadgeRole]:
    def __init__(self, data: T, *, client: ClientManager) -> None:
        self._data: T = data
        self._client = client

    @property
    def name(self) -> str:
        """Returns the name of the badge role.

        Returns
        -------
        str
            The name of the badge role.
        """
        return self._data["name"]

    @property
    def icon_url(self) -> str | None:
        """Returns the icon url of the badge role.

        Returns
        -------
        str | None
            The icon url of the badge role.
        """
        return self._data["icon_url"]

    @property
    def display_order(self) -> int:
        """Returns the display order of the badge role.

        Returns
        -------
        int
            The display order of the badge role.
        """
        return self._data["display_order"]


class AvatarDecoration:
    def __init__(self, raw_avatar_decoration: IAvatarDecoration, *, client: ClientManager) -> None:
        self.__raw_avatar_decoration: IAvatarDecoration = raw_avatar_decoration
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        """Returns the id of the avatar decoration.

        Returns
        -------
        str
            The id of the avatar decoration.
        """
        return self.__raw_avatar_decoration["id"]

    @property
    def angle(self) -> int | None:
        """Returns the angle of the avatar decoration.

        Returns
        -------
        int | None
            The angle of the avatar decoration.
        """
        return self.__raw_avatar_decoration.get("angle")

    @property
    def flip_h(self) -> bool | None:
        """Returns whether the avatar decoration is flipped horizontally.

        Returns
        -------
        bool | None
            Whether the avatar decoration is flipped horizontally.
        """
        return self.__raw_avatar_decoration.get("flip_h")

    @property
    def url(self) -> str:
        """Returns the url of the avatar decoration.

        Returns
        -------
        str
            The url of the avatar decoration.
        """
        return self.__raw_avatar_decoration["url"]

    @property
    def offset_x(self) -> int | None:
        """Returns the x offset of the avatar decoration.

        Returns
        -------
        int | None
            The x offset of the avatar decoration.
        """
        return self.__raw_avatar_decoration.get("offset_x")

    @property
    def offset_y(self) -> int | None:
        """Returns the y offset of the avatar decoration.

        Returns
        -------
        int | None
            The y offset of the avatar decoration.
        """
        return self.__raw_avatar_decoration.get("offset_y")


class PartialUser[PU: IPartialUser]:
    def __init__(self, raw_user: PU, *, client: ClientManager) -> None:
        self._raw_user: PU = raw_user
        self._client: ClientManager = client

    @property
    def id(self) -> str:
        """Returns the id of the user."""
        return self._raw_user["id"]

    @deprecated
    @property
    def nickname(self) -> str | None:
        """Returns the nickname of the user.

        .. deprecated:: 0.6.0
            Use :meth:`mipac.models.lite.user.PartialUser.name` instead.

        Returns
        -------
        str | None
            The nickname of the user.
        """
        return self._raw_user["name"]

    @property
    def name(self) -> str | None:
        """Returns the nickname of the user.

        Returns
        -------
        str | None
            The nickname of the user.
        """
        return self._raw_user["name"]

    @property
    def username(self) -> str:
        """Returns the username of the user.

        Returns
        -------
        str
            The username of the user.
        """
        return self._raw_user["username"]

    @property
    def host(self) -> str | None:
        """Returns the host of the user.

        Returns
        -------
        str | None
            The host of the user.
        """
        return self._raw_user["host"]

    @property
    def avatar_url(self) -> str | None:
        """Returns the avatar url of the user.

        Returns
        -------
        str | None
            The avatar url of the user.
        """
        return self._raw_user["avatar_url"]

    @property
    def avatar_blurhash(self) -> str | None:
        """Returns the avatar blurhash of the user.

        Returns
        -------
        str | None
            The avatar blurhash of the user.
        """
        return self._raw_user["avatar_blurhash"]

    @property
    def avatar_decoration(self) -> list[AvatarDecoration]:
        """Returns the avatar decoration of the user.

        Returns
        -------
        list[AvatarDecoration]
            The avatar decoration of the user.
        """
        return [
            AvatarDecoration(data, client=self._client)
            for data in self._raw_user["avatar_decorations"]
        ]

    @property
    def is_bot(self) -> bool | None:
        """Returns whether the user is a bot.

        Returns
        -------
        bool | None
            Whether the user is a bot.
        """
        return self._raw_user.get("is_bot")

    @property
    def is_cat(self) -> bool | None:
        """Returns whether the user is a cat.

        Returns
        -------
        bool | None
            Whether the user is a cat.
        """
        return self._raw_user.get("is_cat")

    @property
    def instance(self) -> LiteInstance | None:
        """Returns the instance of the user.

        Returns
        -------
        LiteInstance | None
            The instance of the user.
        """
        raw_instance = self._raw_user.get("instance")
        return LiteInstance(raw_instance) if raw_instance else None

    @property
    def emojis(self) -> dict[str, str]:
        """Returns the emojis of the user.

        Returns
        -------
        dict[str, str]
            The emojis of the user.
        """
        return self._raw_user["emojis"]

    @property
    def online_status(self) -> IUserOnlineStatus:
        """Returns the online status of the user.

        Returns
        -------
        IUserOnlineStatus
            The online status of the user.
        """
        return self._raw_user["online_status"]

    @property
    def badge_roles(self) -> list[BadgeRole] | None:
        """Returns the badge roles of the user.

        Returns
        -------
        list[BadgeRole] | None
            The badge roles of the user.
        """
        badge_roles = self._raw_user.get("badge_roles")
        if badge_roles is None:
            return None
        return [BadgeRole(data, client=self._client) for data in badge_roles]

    @property
    def api(self) -> ClientUserManager:
        """Returns the user manager instance.

        Returns
        -------
        ClientUserManager
            The user manager instance
        """
        return self._client._create_client_user_manager(self)

    @property
    def _get_mention(self) -> str:  # TODO: モデルに移す
        """対象のユーザーのメンションを取得します

        Returns
        -------
        str
            メンション
        """
        return f"@{self.username}@{self.host}" if self.instance else f"@{self.username}"

    def _get(self, key: str) -> Any | None:
        return self._raw_user.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, PartialUser) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



import asyncio
import uuid
from typing import Any, Literal, NotRequired, TypedDict
from urllib.parse import urlencode, urlparse, urlunparse

import aiohttp

from mipac.types.permission import Permissions
from mipac.types.user import IUserDetailed
from mipac.utils.format import remove_dict_empty, upper_to_lower


class IMiAuthPayload(TypedDict):
    """MiAuthの戻り値"""

    ok: bool
    token: NotRequired[str]
    user: NotRequired[IUserDetailed]


class MiAuth:
    """Misskey v12以降のインスタンスで使用可能な認証方式です"""

    def __init__(self, host: str, protocol: Literal["http", "https"]) -> None:
        self.host: str = host
        self.protocol: Literal["http", "https"] = protocol
        self.session: str

        self.set_session()

    @property
    def __url(self) -> str:
        return f"{self.protocol}://{self.host}"

    def set_session(self) -> None:
        """sessionを生成してインスタンス変数にセットします"""
        self.session = uuid.uuid4().hex

    async def gen_session(
        self,
        name: str | None = None,
        icon: str | None = None,
        callback: str | None = None,
        permission: list[Permissions] | None = None,
    ) -> str:
        """MiAuthのセッションを生成します

        Parameters
        ----------
        name : str, optional
            セッション名, by default None
        icon : str, optional
            アイコンのURL, by default None
        callback : str, optional
            コールバックURL, by default None
        permission : list[Permissions], optional
            許可する権限, by default None

        Returns
        -------
        str
            生成された認証用URL
        """
        # sessionは使いまわししてはいけないのでこのタイミングで新しいsessionを生成する
        self.set_session()

        query = remove_dict_empty(
            {
                "name": name,
                "icon": icon,
                "callback": callback,
                "permission": ",".join(permission) if permission else None,
            }
        )

        url = urlparse(f"{self.__url}/miauth/{self.session}")
        return urlunparse(
            (url.scheme, url.netloc, url.path, url.params, urlencode(query), url.fragment)
        )

    async def check_session(self) -> Any:
        """MiAuthのセッションが完了したか確認します

        Returns
        -------
        Any
            MiAuthの戻り値
        """
        async with aiohttp.ClientSession() as session:
            res = await session.post(
                f"{self.__url}/api/miauth/{self.session}/check",
                json={},
                headers={"Content-Type": "application/json"},
            )

            if res.status != 200:
                raise Exception(f"MiAuth Check failed, Status code: {res.status}")

            data = await res.json()

            if data is None:
                raise Exception("MiAuth Check failed, Response is None")
            return upper_to_lower(data)

    async def wait_complete(self) -> IMiAuthPayload:
        """MiAuthのセッションが完了するまで待機します

        Returns
        -------
        IMiAuthPayload
            MiAuthの戻り値
        """
        while True:
            is_complete: IMiAuthPayload = await self.check_session()
            if is_complete["ok"] is True:
                break
            await asyncio.sleep(1)

        return is_complete



from __future__ import annotations

from typing import TYPE_CHECKING, Literal, TypedDict

if TYPE_CHECKING:
    from mipac.types.user import IPartialUser

__all__ = ("NoteReactionPayload", "IReactionAcceptance")

IReactionAcceptance = Literal[
    None,
    "likeOnly",
    "likeOnlyForRemote",
    "nonSensitiveOnly",
    "nonSensitiveOnlyForLocalLikeOnlyForRemote",
]


class NoteReactionPayload(TypedDict):
    id: str
    created_at: str
    user: IPartialUser
    type: str



from .base import APIError


class AccessDeniedError(APIError):
    """Access denied."""


class AlreadyAddedError(APIError):
    """That user has already been added to that list."""


class AlreadyBlockingError(APIError):
    """You are already blocking that user."""


class AlreadyClippedError(APIError):
    """The note has already been clipped."""


class AlreadyEndedError(APIError):
    """That game has already ended."""


class AlreadyExpiredError(APIError):
    """The poll is already expired."""


class AlreadyFavoritedError(APIError):
    """You have not favorited the list."""


class AlreadyFollowingError(APIError):
    """You are already following that user."""


class AlreadyLikedError(APIError):
    """The flash has already been liked."""


class AlreadyMovedError(APIError):
    """Account was already moved to another account."""


class AlreadyMutingError(APIError):
    """You are already muting that user."""


class AlreadyPinnedError(APIError):
    """That note has already been pinned."""


class AlreadyPromotedError(APIError):
    """The note has already promoted."""


class AlreadyReactedError(APIError):
    """You are already reacting to that note."""


class AlreadyVotedError(APIError):
    """You have already voted."""


class AuthenticationFailedError(APIError):
    """Authentication failed. Please ensure your token is correct."""


class AvatarNotAnImageError(APIError):
    """The file specified as an avatar is not an image."""


class BannerNotAnImageError(APIError):
    """The file specified as a banner is not an image."""


class BirthdayDateFormatInvalidError(APIError):
    """Birthday date format is invalid."""


class BlockedError(APIError):
    """You are blocked by that user."""


class BlockeeIsYourselfError(APIError):
    """Blockee is yourself."""


class BlockingError(APIError):
    """You are blocking that user."""


class BothWithRepliesAndWithFilesError(APIError):
    """Specifying both withReplies and withFiles is not supported"""


class CannotCreateAlreadyExpiredPollError(APIError):
    """Poll is already expired."""


class CannotRenoteDueToVisibilityError(APIError):
    """You can not Renote due to target visibility."""


class CannotRenoteOutsideOfChannelError(APIError):
    """Cannot renote outside of channel."""


class CannotRenoteToAPureRenoteError(APIError):
    """You can not Renote a pure Renote."""


class CannotReplyToAnInvisibleNoteError(APIError):
    """You cannot reply to an invisible Note."""


class CannotReplyToAPureRenoteError(APIError):
    """You can not reply to a pure Renote."""


class CannotReplyToSpecifiedVisibilityNoteWithExtendedVisibilityError(APIError):
    """You cannot reply to a specified visibility note with extended visibility."""


class CannotReportTheAdminError(APIError):
    """Cannot report the admin."""


class CannotReportYourselfError(APIError):
    """Cannot report yourself."""


class CanNotDeleteInviteCodeError(APIError):
    """You can't delete this invite code."""


class ContainsProhibitedWordsError(APIError):
    """Cannot post because it contains prohibited words."""


class ContainsTooManyMentionsError(APIError):
    """Cannot post because it exceeds the allowed number of mentions."""


class CredentialRequiredError(APIError):
    """Credential required."""


class DestinationAccountForbidsError(APIError):
    """Destination account doesn't have proper 'Known As' alias, or has already moved."""


class DuplicateNameError(APIError):
    """Duplicate name."""


class EmptyFileError(APIError):
    """That file is empty."""


class ExceededLimitOfCreateInviteCodeError(APIError):
    """You have exceeded the limit for creating an invitation code."""


class ExtResourceHashDidntMatchError(APIError):
    """Hash did not match."""


class ExtResourceReturnedInvalidSchemaError(APIError):
    """External resource returned invalid schema."""


class FailedToResolveRemoteUserError(APIError):
    """Failed to resolve remote user."""


class FolloweeIsYourselfError(APIError):
    """Followee is yourself."""


class FollowerIsYourselfError(APIError):
    """Follower is yourself."""


class FollowRequestNotFoundError(APIError):
    """Follow request not found."""


class ForbiddenError(APIError):
    """Forbidden."""


class ForbiddenToSetYourselfError(APIError):
    """You can't set yourself as your own alias."""


class GtlDisabledError(APIError):
    """Global timeline has been disabled."""


class HasChildFilesOrFoldersError(APIError):
    """This folder has child files or folders."""


class InappropriateError(APIError):
    """Cannot upload the file because it has been determined that it possibly contains inappropriate content."""


class IncorrectPasswordError(APIError):
    """Incorrect password."""


class InternalErrorError(APIError):
    """Internal error occurred. Please contact us if the error persists."""


class InvalidChoiceError(APIError):
    """Choice ID is invalid."""


class InvalidDateTimeError(APIError):
    """Invalid date-time format"""


class InvalidFileNameError(APIError):
    """Invalid file name."""


class InvalidParamError(APIError):
    """Invalid param."""


class InvalidRegexpError(APIError):
    """Invalid Regular Expression."""


class InvalidSeedError(APIError):
    """Provided seed is invalid."""


class InvalidUrlError(APIError):
    """Invalid URL"""


class IsRemoteUserError(APIError):
    """Currently unavailable to display reactions of remote users. See https://github.com/misskey-dev/misskey/issues/12964"""


class IAmAiError(APIError):
    """You sent a request to Ai-chan, Misskey's showgirl, instead of the server."""


class LtlDisabledError(APIError):
    """Local timeline has been disabled."""


class MuteeIsYourselfError(APIError):
    """Mutee is yourself."""


class NameAlreadyExistsError(APIError):
    """Specified name already exists."""


class NotAssignedError(APIError):
    """Not assigned."""


class NotBlockingError(APIError):
    """You are not blocking that user."""


class NotFavoritedError(APIError):
    """You have not marked that note a favorite."""


class NotFollowingError(APIError):
    """The other use is not following you."""


class NotLikedError(APIError):
    """You have not liked that flash."""


class NotMutingError(APIError):
    """You are not muting that user."""


class NotReactedError(APIError):
    """You are not reacting to that note."""


class NotRootForbiddenError(APIError):
    """The root can't migrate."""


class NoFollowRequestError(APIError):
    """No follow request."""


class NoFreeSpaceError(APIError):
    """Cannot upload the file because you have no free space of drive."""


class NoPollError(APIError):
    """The note does not attach a poll."""


class NoSecurityKeyError(APIError):
    """No security key."""


class NoSuchAdError(APIError):
    """No such ad."""


class NoSuchAnnouncementError(APIError):
    """No such announcement."""


class NoSuchAntennaError(APIError):
    """No such antenna."""


class NoSuchAppError(APIError):
    """No such app."""


class NoSuchAvatarError(APIError):
    """No such avatar file."""


class NoSuchBannerError(APIError):
    """No such banner file."""


class NoSuchChannelError(APIError):
    """No such channel."""


class NoSuchClipError(APIError):
    """No such clip."""


class NoSuchEmojiError(APIError):
    """No such emoji."""


class NoSuchFileError(APIError):
    """No such file."""


class NoSuchFlashError(APIError):
    """No such flash."""


class NoSuchFolderError(APIError):
    """No such folder."""


class NoSuchGameError(APIError):
    """No such game."""


class NoSuchHashtagError(APIError):
    """No such hashtag."""


class NoSuchInviteCodeError(APIError):
    """No such invite code."""


class NoSuchKeyError(APIError):
    """No such key."""


class NoSuchListError(APIError):
    """No such list."""


class NoSuchNoteError(APIError):
    """No such note."""


class NoSuchObjectError(APIError):
    """No such object."""


class NoSuchPageError(APIError):
    """No such page."""


class NoSuchParentFolderError(APIError):
    """No such parent folder."""


class NoSuchPostError(APIError):
    """No such post."""


class NoSuchRegistrationError(APIError):
    """No such registration."""


class NoSuchRenoteTargetError(APIError):
    """No such renote target."""


class NoSuchReplyTargetError(APIError):
    """No such reply target."""


class NoSuchRoleError(APIError):
    """No such role."""


class NoSuchSessionError(APIError):
    """No such session."""


class NoSuchUserError(APIError):
    """No such user."""


class NoSuchUserListError(APIError):
    """No such user list."""


class NoSuchWebhookError(APIError):
    """No such webhook."""


class PendingSessionError(APIError):
    """This session is not completed yet."""


class PinLimitExceededError(APIError):
    """You can not pin notes any more."""


class RateLimitExceededError(APIError):
    """Rate limit exceeded. Please try again later."""


class ReactionsNotPublicError(APIError):
    """Reactions of the user is not public."""


class RecursiveNestingError(APIError):
    """It can not be structured like nesting folders recursively."""


class RemoteUserNotAllowedError(APIError):
    """Not allowed to load the remote user's list"""


class RestrictedByRoleError(APIError):
    """This feature is restricted by your role."""


class SameNameEmojiExistsError(APIError):
    """Emoji that have same name already exists."""


class StlDisabledError(APIError):
    """Hybrid timeline has been disabled."""


class TargetIsYourselfError(APIError):
    """Target user is yourself."""


class TooBigFileError(APIError):
    """That file is too big."""


class TooManyAntennasError(APIError):
    """You cannot create antenna any more."""


class TooManyClipsError(APIError):
    """You cannot create clip any more."""


class TooManyClipNotesError(APIError):
    """You cannot add notes to the clip any more."""


class TooManyMutedWordsError(APIError):
    """Too many muted words."""


class TooManyUserlistsError(APIError):
    """You cannot create user list any more."""


class TooManyUsersError(APIError):
    """You can not push users any more."""


class TooManyWebhooksError(APIError):
    """You cannot create webhook any more."""


class TwoFactorNotEnabledError(APIError):
    """2fa not enabled."""


class UnavailableError(APIError):
    """Translate of notes unavailable."""


class UnexpectedFileTypeError(APIError):
    """We need csv file."""


class UriNullError(APIError):
    """Local User ActivityPup URI is null."""


class UserIsDeletedError(APIError):
    """User is deleted."""


class UserNotFoundError(APIError):
    """User not found."""


class YourFlashError(APIError):
    """You cannot like your flash."""


class YourPageError(APIError):
    """You cannot like your page."""


class YourPostError(APIError):
    """You cannot like your post."""


class YouHaveBeenBlockedError(APIError):
    """You cannot push this user because you have been blocked by this user."""



"""
=======================
        WARNING
=======================
 This file is automatically generated by compiler/endpoints.py.
 If this file is modified and then auto-generated, the changes will be lost.
"""

from typing import Literal

ENDPOINTS = Literal[
    "/api/admin/meta",
    "/api/admin/abuse-user-reports",
    "/api/admin/accounts/create",
    "/api/admin/accounts/delete",
    "/api/admin/accounts/find-by-email",
    "/api/admin/ad/create",
    "/api/admin/ad/delete",
    "/api/admin/ad/list",
    "/api/admin/ad/update",
    "/api/admin/announcements/create",
    "/api/admin/announcements/delete",
    "/api/admin/announcements/list",
    "/api/admin/announcements/update",
    "/api/admin/avatar-decorations/create",
    "/api/admin/avatar-decorations/delete",
    "/api/admin/avatar-decorations/list",
    "/api/admin/avatar-decorations/update",
    "/api/admin/delete-all-files-of-a-user",
    "/api/admin/unset-user-avatar",
    "/api/admin/unset-user-banner",
    "/api/admin/drive/clean-remote-files",
    "/api/admin/drive/cleanup",
    "/api/admin/drive/files",
    "/api/admin/drive/show-file",
    "/api/admin/emoji/add-aliases-bulk",
    "/api/admin/emoji/add",
    "/api/admin/emoji/copy",
    "/api/admin/emoji/delete-bulk",
    "/api/admin/emoji/delete",
    "/api/admin/emoji/import-zip",
    "/api/admin/emoji/list-remote",
    "/api/admin/emoji/list",
    "/api/admin/emoji/remove-aliases-bulk",
    "/api/admin/emoji/set-aliases-bulk",
    "/api/admin/emoji/set-category-bulk",
    "/api/admin/emoji/set-license-bulk",
    "/api/admin/emoji/update",
    "/api/admin/federation/delete-all-files",
    "/api/admin/federation/refresh-remote-instance-metadata",
    "/api/admin/federation/remove-all-following",
    "/api/admin/federation/update-instance",
    "/api/admin/get-index-stats",
    "/api/admin/get-table-stats",
    "/api/admin/get-user-ips",
    "/api/admin/invite/create",
    "/api/admin/invite/list",
    "/api/admin/promo/create",
    "/api/admin/queue/clear",
    "/api/admin/queue/deliver-delayed",
    "/api/admin/queue/inbox-delayed",
    "/api/admin/queue/promote",
    "/api/admin/queue/stats",
    "/api/admin/relays/add",
    "/api/admin/relays/list",
    "/api/admin/relays/remove",
    "/api/admin/reset-password",
    "/api/admin/resolve-abuse-user-report",
    "/api/admin/send-email",
    "/api/admin/server-info",
    "/api/admin/show-moderation-logs",
    "/api/admin/show-user",
    "/api/admin/show-users",
    "/api/admin/suspend-user",
    "/api/admin/unsuspend-user",
    "/api/admin/update-meta",
    "/api/admin/delete-account",
    "/api/admin/update-user-note",
    "/api/admin/roles/create",
    "/api/admin/roles/delete",
    "/api/admin/roles/list",
    "/api/admin/roles/show",
    "/api/admin/roles/update",
    "/api/admin/roles/assign",
    "/api/admin/roles/unassign",
    "/api/admin/roles/update-default-policies",
    "/api/admin/roles/users",
    "/api/announcements",
    "/api/antennas/create",
    "/api/antennas/delete",
    "/api/antennas/list",
    "/api/antennas/notes",
    "/api/antennas/show",
    "/api/antennas/update",
    "/api/ap/get",
    "/api/ap/show",
    "/api/app/create",
    "/api/app/show",
    "/api/auth/accept",
    "/api/auth/session/generate",
    "/api/auth/session/show",
    "/api/auth/session/userkey",
    "/api/blocking/create",
    "/api/blocking/delete",
    "/api/blocking/list",
    "/api/channels/create",
    "/api/channels/featured",
    "/api/channels/follow",
    "/api/channels/followed",
    "/api/channels/owned",
    "/api/channels/show",
    "/api/channels/timeline",
    "/api/channels/unfollow",
    "/api/channels/update",
    "/api/channels/favorite",
    "/api/channels/unfavorite",
    "/api/channels/my-favorites",
    "/api/channels/search",
    "/api/charts/active-users",
    "/api/charts/ap-request",
    "/api/charts/drive",
    "/api/charts/federation",
    "/api/charts/instance",
    "/api/charts/notes",
    "/api/charts/user/drive",
    "/api/charts/user/following",
    "/api/charts/user/notes",
    "/api/charts/user/pv",
    "/api/charts/user/reactions",
    "/api/charts/users",
    "/api/clips/add-note",
    "/api/clips/remove-note",
    "/api/clips/create",
    "/api/clips/delete",
    "/api/clips/list",
    "/api/clips/notes",
    "/api/clips/show",
    "/api/clips/update",
    "/api/clips/favorite",
    "/api/clips/unfavorite",
    "/api/clips/my-favorites",
    "/api/drive",
    "/api/drive/files",
    "/api/drive/files/attached-notes",
    "/api/drive/files/check-existence",
    "/api/drive/files/create",
    "/api/drive/files/delete",
    "/api/drive/files/find-by-hash",
    "/api/drive/files/find",
    "/api/drive/files/show",
    "/api/drive/files/update",
    "/api/drive/files/upload-from-url",
    "/api/drive/folders",
    "/api/drive/folders/create",
    "/api/drive/folders/delete",
    "/api/drive/folders/find",
    "/api/drive/folders/show",
    "/api/drive/folders/update",
    "/api/drive/stream",
    "/api/email-address/available",
    "/api/endpoint",
    "/api/endpoints",
    "/api/export-custom-emojis",
    "/api/federation/followers",
    "/api/federation/following",
    "/api/federation/instances",
    "/api/federation/show-instance",
    "/api/federation/update-remote-user",
    "/api/federation/users",
    "/api/federation/stats",
    "/api/following/create",
    "/api/following/delete",
    "/api/following/update",
    "/api/following/update-all",
    "/api/following/invalidate",
    "/api/following/requests/accept",
    "/api/following/requests/cancel",
    "/api/following/requests/list",
    "/api/following/requests/reject",
    "/api/gallery/featured",
    "/api/gallery/popular",
    "/api/gallery/posts",
    "/api/gallery/posts/create",
    "/api/gallery/posts/delete",
    "/api/gallery/posts/like",
    "/api/gallery/posts/show",
    "/api/gallery/posts/unlike",
    "/api/gallery/posts/update",
    "/api/get-online-users-count",
    "/api/get-avatar-decorations",
    "/api/hashtags/list",
    "/api/hashtags/search",
    "/api/hashtags/show",
    "/api/hashtags/trend",
    "/api/hashtags/users",
    "/api/i",
    "/api/i/2fa/done",
    "/api/i/2fa/key-done",
    "/api/i/2fa/password-less",
    "/api/i/2fa/register-key",
    "/api/i/2fa/register",
    "/api/i/2fa/update-key",
    "/api/i/2fa/remove-key",
    "/api/i/2fa/unregister",
    "/api/i/apps",
    "/api/i/authorized-apps",
    "/api/i/claim-achievement",
    "/api/i/change-password",
    "/api/i/delete-account",
    "/api/i/export-blocking",
    "/api/i/export-following",
    "/api/i/export-mute",
    "/api/i/export-notes",
    "/api/i/export-clips",
    "/api/i/export-favorites",
    "/api/i/export-user-lists",
    "/api/i/export-antennas",
    "/api/i/favorites",
    "/api/i/gallery/likes",
    "/api/i/gallery/posts",
    "/api/i/import-blocking",
    "/api/i/import-following",
    "/api/i/import-muting",
    "/api/i/import-user-lists",
    "/api/i/import-antennas",
    "/api/i/notifications",
    "/api/i/notifications-grouped",
    "/api/i/page-likes",
    "/api/i/pages",
    "/api/i/pin",
    "/api/i/read-all-unread-notes",
    "/api/i/read-announcement",
    "/api/i/regenerate-token",
    "/api/i/registry/get-all",
    "/api/i/registry/get-detail",
    "/api/i/registry/get",
    "/api/i/registry/keys-with-type",
    "/api/i/registry/keys",
    "/api/i/registry/remove",
    "/api/i/registry/scopes-with-domain",
    "/api/i/registry/set",
    "/api/i/revoke-token",
    "/api/i/signin-history",
    "/api/i/unpin",
    "/api/i/update-email",
    "/api/i/update",
    "/api/i/move",
    "/api/i/webhooks/create",
    "/api/i/webhooks/list",
    "/api/i/webhooks/show",
    "/api/i/webhooks/update",
    "/api/i/webhooks/delete",
    "/api/invite/create",
    "/api/invite/delete",
    "/api/invite/list",
    "/api/invite/limit",
    "/api/meta",
    "/api/emojis",
    "/api/emoji",
    "/api/miauth/gen-token",
    "/api/mute/create",
    "/api/mute/delete",
    "/api/mute/list",
    "/api/renote-mute/create",
    "/api/renote-mute/delete",
    "/api/renote-mute/list",
    "/api/my/apps",
    "/api/notes",
    "/api/notes/children",
    "/api/notes/clips",
    "/api/notes/conversation",
    "/api/notes/create",
    "/api/notes/delete",
    "/api/notes/favorites/create",
    "/api/notes/favorites/delete",
    "/api/notes/featured",
    "/api/notes/global-timeline",
    "/api/notes/hybrid-timeline",
    "/api/notes/local-timeline",
    "/api/notes/mentions",
    "/api/notes/polls/recommendation",
    "/api/notes/polls/vote",
    "/api/notes/reactions",
    "/api/notes/reactions/create",
    "/api/notes/reactions/delete",
    "/api/notes/renotes",
    "/api/notes/replies",
    "/api/notes/search-by-tag",
    "/api/notes/search",
    "/api/notes/show",
    "/api/notes/state",
    "/api/notes/thread-muting/create",
    "/api/notes/thread-muting/delete",
    "/api/notes/timeline",
    "/api/notes/translate",
    "/api/notes/unrenote",
    "/api/notes/user-list-timeline",
    "/api/notifications/create",
    "/api/notifications/mark-all-as-read",
    "/api/notifications/test-notification",
    "/api/page-push",
    "/api/pages/create",
    "/api/pages/delete",
    "/api/pages/featured",
    "/api/pages/like",
    "/api/pages/show",
    "/api/pages/unlike",
    "/api/pages/update",
    "/api/flash/create",
    "/api/flash/delete",
    "/api/flash/featured",
    "/api/flash/like",
    "/api/flash/show",
    "/api/flash/unlike",
    "/api/flash/update",
    "/api/flash/my",
    "/api/flash/my-likes",
    "/api/ping",
    "/api/pinned-users",
    "/api/promo/read",
    "/api/roles/list",
    "/api/roles/show",
    "/api/roles/users",
    "/api/roles/notes",
    "/api/request-reset-password",
    "/api/reset-db",
    "/api/reset-password",
    "/api/server-info",
    "/api/stats",
    "/api/sw/show-registration",
    "/api/sw/update-registration",
    "/api/sw/register",
    "/api/sw/unregister",
    "/api/test",
    "/api/username/available",
    "/api/users",
    "/api/users/clips",
    "/api/users/followers",
    "/api/users/following",
    "/api/users/gallery/posts",
    "/api/users/get-frequently-replied-users",
    "/api/users/featured-notes",
    "/api/users/lists/create",
    "/api/users/lists/delete",
    "/api/users/lists/list",
    "/api/users/lists/pull",
    "/api/users/lists/push",
    "/api/users/lists/show",
    "/api/users/lists/favorite",
    "/api/users/lists/unfavorite",
    "/api/users/lists/update",
    "/api/users/lists/create-from-public",
    "/api/users/lists/update-membership",
    "/api/users/lists/get-memberships",
    "/api/users/notes",
    "/api/users/pages",
    "/api/users/flashs",
    "/api/users/reactions",
    "/api/users/recommendation",
    "/api/users/relation",
    "/api/users/report-abuse",
    "/api/users/search-by-username-and-host",
    "/api/users/search",
    "/api/users/show",
    "/api/users/achievements",
    "/api/users/update-memo",
    "/api/fetch-rss",
    "/api/fetch-external-resources",
    "/api/retention",
    "/api/bubble-game/register",
    "/api/bubble-game/ranking",
    "/api/reversi/cancel-match",
    "/api/reversi/games",
    "/api/reversi/match",
    "/api/reversi/invitations",
    "/api/reversi/show-game",
    "/api/reversi/surrender",
    "/api/reversi/verify",
]



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.poll import ClientPollActions, PollActions

if TYPE_CHECKING:
    from mipac.http import HTTPClient
    from mipac.manager.client import ClientManager


class ClientPollManager(AbstractManager):
    def __init__(self, note_id: str, *, session: HTTPClient, client: ClientManager):
        self.__note_id: str = note_id
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: ClientPollActions = ClientPollActions(
            note_id=self.__note_id, session=self.__session, client=self.__client
        )

    @property
    def action(self) -> ClientPollActions:
        return self.__action


class PollManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: PollActions = PollActions(session=self.__session, client=self.__client)

    @property
    def action(self) -> PollActions:
        return self.__action



"""MiPACで使用する設定を保持するクラスを定義するモジュール"""

from dataclasses import dataclass
from typing import Self


@dataclass
class CacheConfigData:
    """キャッシュの設定を保持するクラス"""

    maxsize: int = 1024
    ttl: int = 360


class CacheConfig:
    """キャッシュの設定を保持するクラス"""

    def __init__(self, options: CacheConfigData) -> None:
        self.maxsize: int = options.maxsize
        self.ttl: int = options.ttl


class Config:
    """MiPACで使用する設定を保持するクラス"""

    def __init__(
        self,
        *,
        host: str = "",
        is_ssl: bool = True,
        cache: CacheConfigData | None = None,
    ) -> None:
        self.is_ssl: bool = is_ssl
        self.host: str = host
        self.cache: CacheConfig = CacheConfig(cache or CacheConfigData())

    def from_dict(
        self,
        *,
        host: str | None = None,
        is_ssl: bool | None = None,
        cache: CacheConfigData | None = None,
    ) -> Self:
        """dictから設定を更新します

        Parameters
        ----------
        host : str | None, optional
            サーバーのhost, by default None
        is_ssl : bool | None, optional
            サーバーがsslかどうか, by default None
        cache : CacheConfigData | None, optional
            キャッシュの設定, by default None

        Returns
        -------
        Self
            更新されたConfigクラス
        """
        self.host = host or self.host
        self.is_ssl = is_ssl if is_ssl is not None else self.is_ssl
        if cache:
            self.cache = CacheConfig(cache)
        return self


config = Config()



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.user import ClientUserActions, UserActions
from mipac.http import HTTPClient
from mipac.manager.admins.user import AdminUserManager, ClientAdminUserManager
from mipac.manager.blocking import BlockingManager, ClientBlockingManager
from mipac.manager.follow import ClientFollowManager, FollowManager
from mipac.manager.users.list import (
    ClientPartialUserListManager,
    ClientUserListManager,
    UserListManager,
)
from mipac.manager.users.mute import ClientMuteManager, MuteManager

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager
    from mipac.models.lite.user import PartialUser


__all__ = ("UserManager", "ClientUserManager")


class ClientUserManager(AbstractManager):
    def __init__(self, user: PartialUser, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: ClientUserActions = ClientUserActions(
            user=user, session=session, client=client
        )
        self.follow: ClientFollowManager = ClientFollowManager(
            user_id=user.id, session=session, client=client
        )
        self.mute: ClientMuteManager = ClientMuteManager(
            user_id=user.id, session=session, client=client
        )
        self.block = ClientBlockingManager(user_id=user.id, session=session, client=client)
        self.list = ClientPartialUserListManager(user_id=user.id, session=session, client=client)
        self.admin: ClientAdminUserManager = ClientAdminUserManager(
            user_id=user.id, session=session, client=client
        )

    @property
    def action(self) -> ClientUserActions:
        return self.__action


class UserManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.follow: FollowManager = FollowManager(session=session, client=client)
        self.mute: MuteManager = MuteManager(session=session, client=client)
        self.block = BlockingManager(session=session, client=client)
        self.list = UserListManager(session=session, client=client)
        self.admin: AdminUserManager = AdminUserManager(session=session, client=client)
        self.__actions: UserActions = UserActions(session=session, client=client)

    @property
    def action(self) -> UserActions:
        return self.__actions

    def _create_client_user_list_manager(self, list_id: str) -> ClientUserListManager:
        return ClientUserListManager(list_id=list_id, session=self.__session, client=self.__client)

    def _create_client_blocking_manager(self, user_id: str) -> ClientBlockingManager:
        return ClientBlockingManager(user_id=user_id, session=self.__session, client=self.__client)



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncGenerator, override

from mipac.abstract.action import AbstractAction
from mipac.http import HTTPClient, Route
from mipac.models.announcement import Announcement, AnnouncementDetailed
from mipac.types.announcement import (
    AnnouncementDisplay,
    AnnoucementIcon,
    IAnnouncement,
    IAnnouncementDetailed,
)
from mipac.utils.format import remove_dict_missing
from mipac.utils.pagination import Pagination
from mipac.utils.util import MISSING

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class SharedAdminAnnouncementActions(AbstractAction):
    def __init__(
        self,
        *,
        session: HTTPClient,
        client: ClientManager,
    ) -> None:
        self._session: HTTPClient = session
        self._client: ClientManager = client

    async def delete(self, *, announce_id: str) -> bool:
        """指定したお知らせを削除します

        Parameters
        ----------
        announce_id : str
            削除したいお知らせのID

        Returns
        -------
        bool
            削除に成功した場合はTrue, それ以外はFalse
        """
        res: bool = await self._session.request(
            Route("POST", "/api/admin/announcements/delete"),
            json={"id": announce_id},
            auth=True,
        )
        return res

    async def update(
        self,
        title: str = MISSING,
        text: str = MISSING,
        image_url: str | None = MISSING,
        icon: AnnoucementIcon = MISSING,
        display: AnnouncementDisplay = MISSING,
        for_existing_users: bool = MISSING,
        silence: bool = MISSING,
        need_confirmation_to_read: bool = MISSING,
        is_active: bool = MISSING,
        *,
        announce_id: str,
    ) -> bool:
        """指定したお知らせを更新します

        Parameters
        ----------
        title : str
            お知らせのタイトル
        text : str
            お知らせの本文
        image_url : str, optional
            お知らせの画像URL, by default None
        icon : AnnoucementIcon, optional
            お知らせのアイコン, by default MISSING
        display : AnnouncementDisplay, optional
            お知らせの表示方法, by default MISSING
        for_existing_users : bool, optional
            既存ユーザーに表示するか, by default MISSING
        silence : bool, optional
            お知らせを静かにするか, by default MISSING
        need_confirmation_to_read : bool, optional
            読んだか確認が必要か, by default MISSING
        is_active : bool, optional
            お知らせが有効かどうか, by default MISSING
        announce_id : str
            更新したいお知らせのID

        Returns
        -------
        bool
            更新に成功した場合はTrue, それ以外はFalse
        """
        if image_url and len(image_url) < 1:  # image_urlはNoneの場合があるので別枠でチェック
            raise ValueError("image_url must not be empty")

        body = remove_dict_missing(
            {
                "id": announce_id,
                "title": title,
                "text": text,
                "imageUrl": image_url,
                "icon": icon,
                "display": display,
                "forExistingUsers": for_existing_users,
                "silence": silence,
                "needConfirmationToRead": need_confirmation_to_read,
                "isActive": is_active,
            }
        )

        res: bool = await self._session.request(
            Route("POST", "/api/admin/announcements/update"),
            json=body,
            auth=True,
            remove_none=False,
        )
        return res


class ClientAdminAnnouncementActions(SharedAdminAnnouncementActions):
    def __init__(
        self,
        announce_id: str,
        *,
        session: HTTPClient,
        client: ClientManager,
    ) -> None:
        super().__init__(session=session, client=client)
        self.__announce_id: str = announce_id

    @override
    async def delete(self) -> bool:
        return await super().delete(announce_id=self.__announce_id)

    @override
    async def update(
        self,
        title: str = MISSING,
        text: str = MISSING,
        image_url: str | None = MISSING,
        icon: AnnoucementIcon = MISSING,
        display: AnnouncementDisplay = MISSING,
        for_existing_users: bool = MISSING,
        silence: bool = MISSING,
        need_confirmation_to_read: bool = MISSING,
        is_active: bool = MISSING,
    ) -> bool:
        return await super().update(
            title=title,
            text=text,
            image_url=image_url,
            icon=icon,
            display=display,
            for_existing_users=for_existing_users,
            silence=silence,
            need_confirmation_to_read=need_confirmation_to_read,
            is_active=is_active,
            announce_id=self.__announce_id,
        )


class AdminAnnouncementActions(SharedAdminAnnouncementActions):
    def __init__(
        self,
        *,
        session: HTTPClient,
        client: ClientManager,
    ):
        super().__init__(session=session, client=client)

    async def create(
        self,
        title: str,
        text: str,
        image_url: str | None = None,
        icon: AnnoucementIcon = MISSING,
        display: AnnouncementDisplay = MISSING,
        for_existing_users: bool = MISSING,
        silence: bool = MISSING,
        need_confirmation_to_read: bool = MISSING,
        user_id: str = MISSING,
    ) -> Announcement:
        body = remove_dict_missing(
            {
                "title": title,
                "text": text,
                "imageUrl": image_url,
                "icon": icon,
                "display": display,
                "forExistingUsers": for_existing_users,
                "silence": silence,
                "needConfirmationToRead": need_confirmation_to_read,
                "userId": user_id,
            }
        )
        created_announcement: IAnnouncement = await self._session.request(
            Route("POST", "/api/admin/announcements/create"),
            json=body,
            auth=True,
            lower=True,
            remove_none=False,
        )
        return Announcement(created_announcement, client=self._client)

    async def gets(
        self,
        limit: int = 10,
        since_id: str | None = None,
        until_id: str | None = None,
        get_all: bool = False,
    ) -> AsyncGenerator[AnnouncementDetailed, None]:  # TODO: 戻り値を改めて確認するべき
        if limit > 100:
            raise ValueError("limitは100以下である必要があります")
        if get_all:
            limit = 100

        body = {
            "limit": limit,
            "sinceId": since_id,
            "untilId": until_id,
        }

        pagination = Pagination[IAnnouncementDetailed](
            self._session, Route("POST", "/api/admin/announcements/list"), json=body
        )

        while True:
            res_annonuncement_systems = await pagination.next()
            for res_announcement_system in res_annonuncement_systems:
                yield AnnouncementDetailed(res_announcement_system, client=self._client)

            if get_all is False or pagination.is_final:
                break



from __future__ import annotations
from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.http import HTTPClient
from mipac.actions.admins.accounts import AdminAccountActions

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class AdminAccountManager(AbstractManager):
    def __init__(self, *, session: HTTPClient, client: ClientManager):
        self.__action: AdminAccountActions = AdminAccountActions(session=session, client=client)

    @property
    def action(self) -> AdminAccountActions:
        return self.__action



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Any

from mipac.models.user import UserDetailedNotMe
from mipac.types.mute import IMuting
from mipac.utils.format import str_to_datetime

if TYPE_CHECKING:
    from mipac.manager.client import ClientManager


class Muting:
    def __init__(self, raw_mute_user: IMuting, *, client: ClientManager) -> None:
        self.__raw_mute_user: IMuting = raw_mute_user
        self.__client: ClientManager = client

    @property
    def id(self) -> str:
        return self.__raw_mute_user["id"]

    @property
    def created_at(self) -> str:
        return self.__raw_mute_user["created_at"]

    @property
    def expires_at(self) -> datetime | None:
        return (
            str_to_datetime(self.__raw_mute_user["expires_at"])
            if self.__raw_mute_user["expires_at"]
            else None
        )

    @property
    def mutee_id(self) -> str:
        return self.__raw_mute_user["mutee_id"]

    @property
    def mutee(self) -> UserDetailedNotMe:
        return UserDetailedNotMe(self.__raw_mute_user["mutee"], client=self.__client)

    def _get(self, key: str) -> Any | None:
        return self.__raw_mute_user.get(key)

    def __eq__(self, __value: object) -> bool:
        return isinstance(__value, Muting) and self.id == __value.id

    def __ne__(self, __value: object) -> bool:
        return not self.__eq__(__value)



from __future__ import annotations

from typing import TYPE_CHECKING

from mipac.abstract.manager import AbstractManager
from mipac.actions.admins.roles import AdminRoleActions, ClientAdminRoleActions
from mipac.http import HTTPClient

if TYPE_CHECKING:
    from mipac.client import ClientManager


class ClientAdminRoleManager(AbstractManager):
    def __init__(self, role_id: str, *, session: HTTPClient, client: ClientManager):
        self.__session: HTTPClient = session
        self.__client: ClientManager = client
        self.__action: ClientAdminRoleActions = ClientAdminRoleActions(
            role_id=role_id, session=self.__session, client=self.__client
        )

    @property
    def action(self) -> ClientAdminRoleActions:
        return self.__action


class AdminRoleManager:
    def __init__(self, role_id: str | None = None, *, session: HTTPClient, client: ClientManager):
        self.__action: AdminRoleActions = AdminRoleActions(session=session, client=client)

    @property
    def action(self) -> AdminRoleActions:
        return self.__action



from typing import NotRequired, TypedDict

from mipac.types.drive import IFile
from mipac.types.user import IPartialUser


class IGalleryPost(TypedDict):
    id: str
    created_at: str
    updated_at: str  # 更新が一度もない場合は created_atとほぼ同じものが入ってる
    user_id: str
    user: IPartialUser
    title: str
    description: str | None
    file_ids: list[str]
    files: list[IFile]
    tags: NotRequired[list[str]]  # Misskeyの今の実装では常に存在しない?
    is_sensitive: bool
    liked_count: int
    is_liked: NotRequired[bool]
